@@ -54,7 +54,7 @@ reflow_error = poppler_error if poppler_error else magick_error
 
 pdfreflow_libs = []
 if iswindows:
-    pdfreflow_libs = ['advapi32', 'User32', 'Gdi32']
+    pdfreflow_libs = ['advapi32', 'User32', 'Gdi32', 'zlib']
 
 extensions = [
 
@@ -186,7 +186,7 @@ if isfreebsd:
 
 
 if isosx:
-    x, p = ('i386', 'ppc')
+    x, p = ('i386', 'x86_64')
     archs = ['-arch', x, '-arch', p, '-isysroot',
                 OSX_SDK]
     cflags.append('-D_OSX')
@@ -339,7 +339,7 @@ class Build(Command):
         obj_pat = 'release\\*.obj' if iswindows else '*.o'
         objects = glob.glob(obj_pat)
         if not objects or self.newer(objects, ext.sources+ext.headers):
-            archs = 'x86 ppc'
+            archs = 'x86 x86_64'
             pro = textwrap.dedent('''\
                 TARGET   = %s
                 TEMPLATE = lib
@@ -348,8 +348,12 @@ class Build(Command):
                 VERSION  = 1.0.0
                 CONFIG   += %s
             ''')%(ext.name, ' '.join(ext.headers), ' '.join(ext.sources), archs)
+            pro = pro.replace('\\', '\\\\')
             open(ext.name+'.pro', 'wb').write(pro)
-            subprocess.check_call([QMAKE, '-o', 'Makefile', ext.name+'.pro'])
+            qmc = [QMAKE, '-o', 'Makefile']
+            if iswindows:
+                qmc += ['-spec', 'win32-msvc2008']
+            subprocess.check_call(qmc + [ext.name+'.pro'])
             subprocess.check_call([make, '-f', 'Makefile'])
             objects = glob.glob(obj_pat)
         return list(map(self.a, objects))
