@@ -40,6 +40,7 @@ class LinuxFreeze(Command):
                         '/usr/bin/pdftohtml',
                         '/usr/lib/libwmflite-0.2.so.7',
                         '/usr/lib/liblcms.so.1',
+                        '/usr/lib/liblcms2.so.2',
                         '/usr/lib/libstlport.so.5.1',
                         '/tmp/calibre-mount-helper',
                         '/usr/lib/libunrar.so',
@@ -47,13 +48,12 @@ class LinuxFreeze(Command):
                         '/usr/lib/libsqlite3.so.0',
                         '/usr/lib/libsqlite3.so.0',
                         '/usr/lib/libmng.so.1',
-                        '/usr/lib/libpodofo.so.0.8.1',
+                        '/usr/lib/libpodofo.so.0.8.2',
                         '/lib/libz.so.1',
                         '/lib/libuuid.so.1',
-                        '/usr/lib/libtiff.so.3',
+                        '/usr/lib/libtiff.so.5',
                         '/lib/libbz2.so.1',
-                        '/usr/lib/libpoppler.so.5',
-                        '/usr/lib/libpoppler-qt4.so.3',
+                        '/usr/lib/libpoppler.so.6',
                         '/usr/lib/libxml2.so.2',
                         '/usr/lib/libopenjpeg.so.2',
                         '/usr/lib/libxslt.so.1',
@@ -62,10 +62,10 @@ class LinuxFreeze(Command):
                         '/usr/lib/libgthread-2.0.so.0',
                         stdcpp,
                         ffi,
-                        '/usr/lib/libpng12.so.0',
+                        '/usr/lib/libpng14.so.14',
                         '/usr/lib/libexslt.so.0',
-                        '/usr/lib/libMagickWand.so.2',
-                        '/usr/lib/libMagickCore.so.2',
+                        '/usr/lib/libMagickWand.so.3',
+                        '/usr/lib/libMagickCore.so.3',
                         '/usr/lib/libgcrypt.so.11',
                         '/usr/lib/libgpg-error.so.0',
                         '/usr/lib/libphonon.so.4',
@@ -116,6 +116,10 @@ class LinuxFreeze(Command):
                 glob.glob('src/calibre/gui2/convert/*.py')]
         includes += ['calibre.gui2.catalog.'+x.split('/')[-1].rpartition('.')[0] for x in \
                 glob.glob('src/calibre/gui2/catalog/*.py')]
+        includes += ['calibre.gui2.actions.'+x.split('/')[-1].rpartition('.')[0] for x in \
+                glob.glob('src/calibre/gui2/actions/*.py')]
+        includes += ['calibre.gui2.preferences.'+x.split('/')[-1].rpartition('.')[0] for x in \
+                glob.glob('src/calibre/gui2/preferences/*.py')]
 
 
         LOADER = '/tmp/loader.py'
