@@ -21,8 +21,6 @@ from calibre.constants import iswindows, isosx, islinux, isfreebsd, isfrozen, \
                               filesystem_encoding, plugins, config_dir
 from calibre.startup import winutil, winutilerror
 
-import mechanize
-
 uuid.uuid4() # Imported before PyQt4 to workaround PyQt4 util-linux conflict on gentoo
 
 if False:
@@ -269,7 +267,8 @@ def browser(honor_time=True, max_time=2, mobile_browser=False):
     :param honor_time: If True honors pause time in refresh requests
     :param max_time: Maximum time in seconds to wait during a refresh request
     '''
-    opener = mechanize.Browser()
+    from calibre.utils.browser import Browser
+    opener = Browser()
     opener.set_handle_refresh(True, max_time=max_time, honor_time=honor_time)
     opener.set_handle_robots(False)
     opener.addheaders = [('User-agent', ' Mozilla/5.0 (Windows; U; Windows CE 5.1; rv:1.8.1a3) Gecko/20060610 Minimo/0.016' if mobile_browser else \
@@ -445,6 +444,9 @@ xml_entity_to_unicode = partial(entity_to_unicode, result_exceptions = {
 def replace_entities(raw):
     return _ent_pat.sub(entity_to_unicode, raw)
 
+def xml_replace_entities(raw):
+    return _ent_pat.sub(xml_entity_to_unicode, raw)
+
 def prepare_string_for_xml(raw, attribute=False):
     raw = _ent_pat.sub(entity_to_unicode, raw)
     raw = raw.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
@@ -455,6 +457,24 @@ def prepare_string_for_xml(raw, attribute=False):
 def isbytestring(obj):
     return isinstance(obj, (str, bytes))
 
+def force_unicode(obj, enc=preferred_encoding):
+    if isbytestring(obj):
+        try:
+            obj = obj.decode(enc)
+        except:
+            try:
+                obj = obj.decode(filesystem_encoding if enc ==
+                        preferred_encoding else preferred_encoding)
+            except:
+                try:
+                    obj = obj.decode('utf-8')
+                except:
+                    obj = repr(obj)
+                    if isbytestring(obj):
+                        obj = obj.decode('utf-8')
+    return obj
+
+
 def human_readable(size):
     """ Convert a size in bytes into a human readable form """
     divisor, suffix = 1, "B"
