@@ -1,8 +1,10 @@
+# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 from __future__ import with_statement
 __license__ = 'GPL 3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
+import sys
 from itertools import izip
 from xml.sax.saxutils import escape
 
@@ -248,6 +250,21 @@ class OutputProfile(Plugin):
     #: If True, the date is appended to the title of downloaded news
     periodical_date_in_title = True
 
+    #: Characters used in jackets and catalogs
+	missing_char = u'x'
+    ratings_char = u'*'
+    empty_ratings_char = u' '
+    read_char = u'+'
+
+    #: Unsupported unicode characters to be replaced during preprocessing
+    unsupported_unicode_chars = []
+
+    #: Number of ems that the left margin of a blockquote is rendered as
+    mobi_ems_per_blockquote = 1.0
+
+    #: Special periodical formatting needed in EPUB
+    epub_periodical_format = None
+
     @classmethod
     def tags_to_string(cls, tags):
         return escape(', '.join(tags))
@@ -273,6 +290,12 @@ class iPadOutput(OutputProfile):
             'macros': {'border-width': '{length}|medium|thick|thin'}
         }
     ]
+
+	missing_char = u'\u2715\u200a'		# stylized 'x' plus hair space
+    ratings_char = u'\u2605'			# filled star
+	empty_ratings_char = u'\u2606'		# hollow star
+    read_char = u'\u2713'				# check mark
+
     touchscreen = True
     # touchscreen_news_css {{{
     touchscreen_news_css = u'''
@@ -406,6 +429,13 @@ class iPadOutput(OutputProfile):
         '''
         # }}}
 
+class TabletOutput(iPadOutput):
+    name = 'Tablet'
+    short_name = 'tablet'
+    description = _('Intended for generic tablet devices, does no resizing of images')
+
+    screen_size = (sys.maxint, sys.maxint)
+    comic_screen_size = (sys.maxint, sys.maxint)
 
 class SonyReaderOutput(OutputProfile):
 
@@ -418,6 +448,11 @@ class SonyReaderOutput(OutputProfile):
     dpi                       = 168.451
     fbase                     = 12
     fsizes                    = [7.5, 9, 10, 12, 15.5, 20, 22, 24]
+    unsupported_unicode_chars = [u'\u201f', u'\u201b']
+
+    epub_periodical_format = 'sony'
+    #periodical_date_in_title = False
+
 
 class KoboReaderOutput(OutputProfile):
 
@@ -471,7 +506,6 @@ class SonyReaderLandscapeOutput(SonyReaderOutput):
     screen_size               = (784, 1012)
     comic_screen_size         = (784, 1012)
 
-
 class MSReaderOutput(OutputProfile):
 
     name        = 'Microsoft Reader'
@@ -540,6 +574,8 @@ class CybookOpusOutput(SonyReaderOutput):
     fbase                     = 16
     fsizes                    = [12, 12, 14, 16, 18, 20, 22, 24]
 
+    epub_periodical_format = None
+
 class KindleOutput(OutputProfile):
 
     name        = 'Kindle'
@@ -554,9 +590,16 @@ class KindleOutput(OutputProfile):
     supports_mobi_indexing = True
     periodical_date_in_title = False
 
+	missing_char = u'x\u2009'
+	empty_ratings_char = u'\u2606'
+    ratings_char = u'\u2605'
+    read_char = u'\u2713'
+
+    mobi_ems_per_blockquote = 2.0
+
     @classmethod
     def tags_to_string(cls, tags):
-        return u'%s <br/><span style="color: white">%s</span>' % (', '.join(tags),
+        return u'%s <br/><span style="color:white">%s</span>' % (', '.join(tags),
                 'ttt '.join(tags)+'ttt ')
 
 class KindleDXOutput(OutputProfile):
@@ -568,9 +611,13 @@ class KindleDXOutput(OutputProfile):
     # Screen size is a best guess
     screen_size               = (744, 1022)
     dpi                       = 150.0
-    comic_screen_size         = (741, 1022)
+    comic_screen_size = (771, 1116)
+    #comic_screen_size         = (741, 1022)
     supports_mobi_indexing = True
     periodical_date_in_title = False
+    ratings_char = u'\u2605'
+    read_char = u'\u2713'
+    mobi_ems_per_blockquote = 2.0
 
     @classmethod
     def tags_to_string(cls, tags):
@@ -631,11 +678,26 @@ class NookOutput(OutputProfile):
     fbase                     = 16
     fsizes                    = [12, 12, 14, 16, 18, 20, 22, 24]
 
+class BambookOutput(OutputProfile):
+
+    author      = 'Li Fanxi'
+    name        = 'Sanda Bambook'
+    short_name  = 'bambook'
+    description = _('This profile is intended for the Sanda Bambook.')
+
+    # Screen size is a best guess
+    screen_size               = (600, 800)
+    comic_screen_size         = (540, 700)
+    dpi                       = 168.451
+    fbase                     = 12
+    fsizes                    = [10, 12, 14, 16]
+
 output_profiles = [OutputProfile, SonyReaderOutput, SonyReader300Output,
         SonyReader900Output, MSReaderOutput, MobipocketOutput, HanlinV3Output,
         HanlinV5Output, CybookG3Output, CybookOpusOutput, KindleOutput,
-        iPadOutput, KoboReaderOutput,
+        iPadOutput, KoboReaderOutput, TabletOutput,
         SonyReaderLandscapeOutput, KindleDXOutput, IlliadOutput,
-        IRexDR1000Output, IRexDR800Output, JetBook5Output, NookOutput,]
+        IRexDR1000Output, IRexDR800Output, JetBook5Output, NookOutput,
+        BambookOutput, ]
 
 output_profiles.sort(cmp=lambda x,y:cmp(x.name.lower(), y.name.lower()))
