@@ -37,7 +37,11 @@ class DevicePlugin(Plugin):
     THUMBNAIL_HEIGHT = 68
 
     #: Whether the metadata on books can be set via the GUI.
-    CAN_SET_METADATA = True
+    CAN_SET_METADATA = ['title', 'authors', 'collections']
+
+    # Set this to None if the books on the device are files that the GUI can
+    # access in order to add the books from the device to the library
+    BACKLOADING_ERROR_MESSAGE = _('Cannot get files from this device')
 
     #: Path separator for paths to books on device
     path_sep = os.sep
@@ -70,9 +74,9 @@ class DevicePlugin(Plugin):
         if bcd is None or len(bcd) == 0:
             return True
         for c in bcd:
-            # Bug in winutil.get_usb_devices converts a to :
-            rev = ('rev_%4.4x'%c).replace('a', ':')
-            if rev in device_id:
+            rev = 'rev_%4.4x'%c
+            # Bug in winutil.get_usb_devices sometimes converts a to :
+            if rev in device_id or rev.replace('a', ':') in device_id:
                 return True
         return False
 
@@ -316,7 +320,7 @@ class DevicePlugin(Plugin):
                       being uploaded to the device.
         :param names: A list of file names that the books should have
                       once uploaded to the device. len(names) == len(files)
-        :param metadata: If not None, it is a list of :class:`MetaInformation` objects.
+        :param metadata: If not None, it is a list of :class:`Metadata` objects.
                          The idea is to use the metadata to determine where on the device to
                          put the book. len(metadata) == len(files). Apart from the regular
                          cover (path to cover), there may also be a thumbnail attribute, which should
@@ -335,7 +339,7 @@ class DevicePlugin(Plugin):
         the device.
 
         :param locations: Result of a call to L{upload_books}
-        :param metadata: List of :class:`MetaInformation` objects, same as for
+        :param metadata: List of :class:`Metadata` objects, same as for
                          :meth:`upload_books`.
         :param booklists: A tuple containing the result of calls to
                           (:meth:`books(oncard=None)`,
@@ -411,6 +415,24 @@ class DevicePlugin(Plugin):
         '''
         raise NotImplementedError()
 
+    def set_plugboards(self, plugboards, pb_func):
+        '''
+        provide the driver the current set of plugboards and a function to
+        select a specific plugboard. This method is called immediately before
+        add_books and sync_booklists.
+
+        pb_func is a callable with the following signature::
+            def pb_func(device_name, format, plugboards)
+
+        You give it the current device name (either the class name or
+        DEVICE_PLUGBOARD_NAME), the format you are interested in (a 'real'
+        format or 'device_db'), and the plugboards (you were given those by
+        set_plugboards, the same place you got this method).
+
+        :return: None or a single plugboard instance.
+
+        '''
+        pass
 
 class BookList(list):
     '''
