@@ -72,6 +72,15 @@ class SWEEX(USBMS):
     EBOOK_DIR_MAIN = ''
     SUPPORTS_SUB_DIRS = True
 
+class Q600(SWEEX):
+
+    name = 'Digma Q600 Device interface'
+    gui_name = 'Q600'
+    description    = _('Communicate with the Digma Q600')
+
+    BCD = [0x325]
+    FORMATS     = ['epub', 'fb2', 'mobi', 'prc', 'html', 'rtf', 'chm', 'pdf', 'txt']
+
 class KOGAN(SWEEX):
 
     name           = 'Kogan Device Interface'
@@ -102,12 +111,49 @@ class PDNOVEL(USBMS):
     DELETE_EXTS = ['.jpg', '.jpeg', '.png']
 
 
-    def upload_cover(self, path, filename, metadata):
+    def upload_cover(self, path, filename, metadata, filepath):
         coverdata = getattr(metadata, 'thumbnail', None)
         if coverdata and coverdata[2]:
             with open('%s.jpg' % os.path.join(path, filename), 'wb') as coverfile:
                 coverfile.write(coverdata[2])
 
+class PDNOVEL_KOBO(PDNOVEL):
+    name = 'Pandigital Kobo device interface'
+    gui_name = 'PD Novel (Kobo)'
+    description = _('Communicate with the Pandigital Novel')
+
+    BCD         = [0x222]
+
+    EBOOK_DIR_MAIN = 'eBooks'
+
+    def upload_cover(self, path, filename, metadata, filepath):
+        coverdata = getattr(metadata, 'thumbnail', None)
+        if coverdata and coverdata[2]:
+            dirpath = os.path.join(path, '.thumbnail')
+            if not os.path.exists(dirpath):
+                os.makedirs(dirpath)
+            with open(os.path.join(dirpath, filename+'.jpg'), 'wb') as coverfile:
+                coverfile.write(coverdata[2])
+
+
+class VELOCITYMICRO(USBMS):
+    name = 'VelocityMicro device interface'
+    gui_name = 'VelocityMicro'
+    description = _('Communicate with the VelocityMicro')
+    author = 'Kovid Goyal'
+    supported_platforms = ['windows', 'linux', 'osx']
+    FORMATS = ['epub', 'pdb', 'txt', 'html', 'pdf']
+
+    VENDOR_ID   = [0x18d1]
+    PRODUCT_ID  = [0xb015]
+    BCD         = [0x224]
+
+    VENDOR_NAME = 'ANDROID'
+    WINDOWS_MAIN_MEM = WINDOWS_CARD_A_MEM = '__UMS_COMPOSITE'
+
+    EBOOK_DIR_MAIN = 'eBooks'
+    SUPPORTS_SUB_DIRS = False
+
 class GEMEI(USBMS):
     name           = 'Gemei Device Interface'
     gui_name       = 'GM2000'
