@@ -14,7 +14,7 @@ from calibre.ebooks.conversion.preprocess import HTMLPreProcessor
 from calibre.ptempfile import PersistentTemporaryDirectory
 from calibre.utils.date import parse_date
 from calibre.utils.zipfile import ZipFile
-from calibre import extract, walk
+from calibre import extract, walk, isbytestring, filesystem_encoding
 from calibre.constants import __version__
 
 DEBUG_README=u'''
@@ -77,6 +77,10 @@ class Plumber(object):
         :param input: Path to input file.
         :param output: Path to output file/directory
         '''
+        if isbytestring(input):
+            input = input.decode(filesystem_encoding)
+        if isbytestring(output):
+            output = output.decode(filesystem_encoding)
         self.original_input_arg = input
         self.input = os.path.abspath(input)
         self.output = os.path.abspath(output)
@@ -241,7 +245,7 @@ OptionRecommendation(name='toc_filter',
 
 OptionRecommendation(name='chapter',
         recommended_value="//*[((name()='h1' or name()='h2') and "
-              r"re:test(., 'chapter|book|section|part\s+', 'i')) or @class "
+              r"re:test(., 'chapter|book|section|part|prologue|epilogue\s+', 'i')) or @class "
               "= 'chapter']", level=OptionRecommendation.LOW,
             help=_('An XPath expression to detect chapter titles. The default '
                 'is to consider <h1> or <h2> tags that contain the words '
@@ -362,6 +366,23 @@ OptionRecommendation(name='preprocess_html',
             )
         ),
 
+OptionRecommendation(name='html_unwrap_factor',
+        recommended_value=0.40, level=OptionRecommendation.LOW,
+        help=_('Scale used to determine the length at which a line should '
+            'be unwrapped if preprocess is enabled. Valid values are a decimal between 0 and 1. The '
+            'default is 0.40, just below the median line length. This will unwrap typical books '
+            ' with hard line breaks, but should be reduced if the line length is variable.'
+            )
+        ),
+
+OptionRecommendation(name='smarten_punctuation',
+        recommended_value=False, level=OptionRecommendation.LOW,
+        help=_('Convert plain quotes, dashes and ellipsis to their '
+            'typographically correct equivalents. For details, see '
+            'http://daringfireball.net/projects/smartypants'
+            )
+        ),
+
 OptionRecommendation(name='remove_header',
         recommended_value=False, level=OptionRecommendation.LOW,
         help=_('Use a regular expression to try and remove the header.'
@@ -684,13 +705,13 @@ OptionRecommendation(name='timestamp',
                                             self.opts.read_metadata_from_opf)
             opf = OPF(open(self.opts.read_metadata_from_opf, 'rb'),
                       os.path.dirname(self.opts.read_metadata_from_opf))
-            mi = MetaInformation(opf)
+            mi = opf.to_book_metadata()
         self.opts_to_mi(mi)
         if mi.cover:
             if mi.cover.startswith('http:') or mi.cover.startswith('https:'):
                 mi.cover = self.download_cover(mi.cover)
             ext = mi.cover.rpartition('.')[-1].lower().strip()
-            if ext not in ('png', 'jpg', 'jpeg'):
+            if ext not in ('png', 'jpg', 'jpeg', 'gif'):
                 ext = 'jpg'
             mi.cover_data = (ext, open(mi.cover, 'rb').read())
             mi.cover = None
@@ -821,7 +842,8 @@ OptionRecommendation(name='timestamp',
                 self.opts_to_mi(self.user_metadata)
             if not hasattr(self.oeb, 'manifest'):
                 self.oeb = create_oebbook(self.log, self.oeb, self.opts,
-                        self.input_plugin)
+                        self.input_plugin,
+                        encoding=self.input_plugin.output_encoding)
             self.input_plugin.postprocess_book(self.oeb, self.opts, self.log)
             self.opts.is_image_collection = self.input_plugin.is_image_collection
             pr = CompositeProgressReporter(0.34, 0.67, self.ui_reporter)
