@@ -27,6 +27,8 @@ TABLE_TAGS = set(['table', 'tr', 'td', 'th', 'caption'])
 SPECIAL_TAGS = set(['hr', 'br'])
 CONTENT_TAGS = set(['img', 'hr', 'br'])
 
+NOT_VTAGS = HEADER_TAGS | NESTABLE_TAGS | TABLE_TAGS | SPECIAL_TAGS | \
+    CONTENT_TAGS
 PAGE_BREAKS = set(['always', 'left', 'right'])
 
 COLLAPSE = re.compile(r'[ \t\r\n\v]+')
@@ -57,8 +59,6 @@ class FormatState(object):
         self.indent = 0.
         self.fsize = 3
         self.ids = set()
-        self.valign = 'baseline'
-        self.nest = False
         self.italic = False
         self.bold = False
         self.strikethrough = False
@@ -76,7 +76,6 @@ class FormatState(object):
                and self.italic == other.italic \
                and self.bold == other.bold \
                and self.href == other.href \
-               and self.valign == other.valign \
                and self.preserve == other.preserve \
                and self.family == other.family \
                and self.bgcolor == other.bgcolor \
@@ -184,13 +183,14 @@ class MobiMLizer(object):
             elif tag in NESTABLE_TAGS and istate.rendered:
                 para = wrapper = bstate.nested[-1]
             elif left > 0 and indent >= 0:
+                ems = self.profile.mobi_ems_per_blockquote
                 para = wrapper = etree.SubElement(parent, XHTML('blockquote'))
                 para = wrapper
-                emleft = int(round(left / self.profile.fbase)) - 1
+                emleft = int(round(left / self.profile.fbase)) - ems
                 emleft = min((emleft, 10))
-                while emleft > 0:
+                while emleft > ems/2.0:
                     para = etree.SubElement(para, XHTML('blockquote'))
-                    emleft -= 1
+                    emleft -= ems
             else:
                 para = wrapper = etree.SubElement(parent, XHTML('p'))
             bstate.inline = bstate.para = para
@@ -223,7 +223,6 @@ class MobiMLizer(object):
             return
         if not pstate or istate != pstate:
             inline = para
-            valign = istate.valign
             fsize = istate.fsize
             href = istate.href
             if not href:
@@ -233,19 +232,8 @@ class MobiMLizer(object):
             else:
                 inline = etree.SubElement(inline, XHTML('a'), href=href)
                 bstate.anchor = inline
-            if valign == 'super':
-                parent = inline
-                if istate.nest and bstate.inline is not None:
-                    parent = bstate.inline
-                    istate.nest = False
-                inline = etree.SubElement(parent, XHTML('sup'))
-            elif valign == 'sub':
-                parent = inline
-                if istate.nest and bstate.inline is not None:
-                    parent = bstate.inline
-                    istate.nest = False
-                inline = etree.SubElement(parent, XHTML('sub'))
-            elif fsize != 3:
+
+            if fsize != 3:
                 inline = etree.SubElement(inline, XHTML('font'),
                                           size=str(fsize))
             if istate.family == 'monospace':
@@ -278,7 +266,8 @@ class MobiMLizer(object):
             else:
                 inline.append(item)
 
-    def mobimlize_elem(self, elem, stylizer, bstate, istates):
+    def mobimlize_elem(self, elem, stylizer, bstate, istates,
+            ignore_valign=False):
         if not isinstance(elem.tag, basestring) \
            or namespace(elem.tag) != XHTML_NS:
             return
@@ -286,7 +275,15 @@ class MobiMLizer(object):
         # <mbp:frame-set/> does not exist lalalala
         if style['display'] in ('none', 'oeb-page-head', 'oeb-page-foot') \
            or style['visibility'] == 'hidden':
-            return
+            id_ = elem.get('id', None)
+            if id_:
+                # Keep anchors so people can use display:none
+                # to generate hidden TOCs
+                elem.clear()
+                elem.text = None
+                elem.set('id', id_)
+            else:
+                return
         tag = barename(elem.tag)
         istate = copy.copy(istates[-1])
         istate.rendered = False
@@ -350,15 +347,6 @@ class MobiMLizer(object):
             istate.family = 'sans-serif'
         else:
             istate.family = 'serif'
-        valign = style['vertical-align']
-        if valign in ('super', 'text-top') or asfloat(valign) > 0:
-            istate.nest = istate.valign in ('sub', 'super')
-            istate.valign = 'super'
-        elif valign == 'sub'  or asfloat(valign) < 0:
-            istate.nest = istate.valign in ('sub', 'super')
-            istate.valign = 'sub'
-        else:
-            istate.valign = 'baseline'
         if 'id' in elem.attrib:
             istate.ids.add(elem.attrib['id'])
         if 'name' in elem.attrib:
@@ -375,11 +363,15 @@ class MobiMLizer(object):
                     if value == getattr(self.profile, prop):
                         result = '100%'
                     else:
+                        # Amazon's renderer does not support
+                        # img sizes in units other than px
+                        # See #7520 for test case
                         try:
-                            ems = int(round(float(value) / self.profile.fbase))
+                            pixs = int(round(float(value) / \
+                                (72./self.profile.dpi)))
                         except:
                             continue
-                        result = "%dem" % ems
+                        result = "%d"%pixs
                     istate.attrib[prop] = result
         elif tag == 'hr' and asfloat(style['width']) > 0:
             prop = style['width'] / self.profile.width
@@ -398,6 +390,15 @@ class MobiMLizer(object):
             for attr in ('rowspan', 'colspan','width','border','scope'):
                 if attr in elem.attrib:
                     istate.attrib[attr] = elem.attrib[attr]
+        if tag == 'q':
+            t = elem.text
+            if not t:
+                t = ''
+            elem.text = u'\u201c' + t
+            t = elem.tail
+            if not t:
+                t = ''
+            elem.tail = u'\u201d' + t
         text = None
         if elem.text:
             if istate.preserve:
@@ -406,6 +407,36 @@ class MobiMLizer(object):
                 text = None
             else:
                 text = COLLAPSE.sub(' ', elem.text)
+        valign = style['vertical-align']
+        not_baseline = valign in ('super', 'sub', 'text-top',
+                'text-bottom')
+        vtag = 'sup' if valign in ('super', 'text-top') else 'sub'
+        if not_baseline and not ignore_valign and tag not in NOT_VTAGS and not isblock:
+            nroot = etree.Element(XHTML('html'), nsmap=MOBI_NSMAP)
+            vbstate = BlockState(etree.SubElement(nroot, XHTML('body')))
+            vbstate.para = etree.SubElement(vbstate.body, XHTML('p'))
+            self.mobimlize_elem(elem, stylizer, vbstate, istates,
+                    ignore_valign=True)
+            if len(istates) > 0:
+                istates.pop()
+            if len(istates) == 0:
+                istates.append(FormatState())
+            at_start = bstate.para is None
+            if at_start:
+                self.mobimlize_content('span', '', bstate, istates)
+            parent = bstate.para if bstate.inline is None else bstate.inline
+            if parent is not None:
+                vtag = etree.SubElement(parent, XHTML(vtag))
+                # Add anchors
+                for child in vbstate.body:
+                    if child is not vbstate.para:
+                        vtag.append(child)
+                    else:
+                        break
+                for child in vbstate.para:
+                    vtag.append(child)
+                return
+
         if text or tag in CONTENT_TAGS or tag in NESTABLE_TAGS:
             self.mobimlize_content(tag, text, bstate, istates)
         for child in elem:
@@ -420,6 +451,8 @@ class MobiMLizer(object):
                     tail = COLLAPSE.sub(' ', child.tail)
             if tail:
                 self.mobimlize_content(tag, tail, bstate, istates)
+
+
         if bstate.content and style['page-break-after'] in PAGE_BREAKS:
             bstate.pbreak = True
         if isblock:
