@@ -221,7 +221,10 @@ class MetadataHeader(BookHeader):
         else:
             end = self.section_offset(number + 1)
         self.stream.seek(start)
-        return self.stream.read(end - start)
+        try:
+            return self.stream.read(end - start)
+        except OverflowError:
+            return self.stream.read(os.stat(self.stream.name).st_size - start)
 
 
 class MobiReader(object):
@@ -234,7 +237,7 @@ class MobiReader(object):
         self.debug = debug
         self.embedded_mi = None
         self.base_css_rules = textwrap.dedent('''
-                blockquote { margin: 0em 0em 0em 1.25em; text-align: justify }
+                blockquote { margin: 0em 0em 0em 2em; text-align: justify }
 
                 p { margin: 0em; text-align: justify }
 
@@ -398,6 +401,8 @@ class MobiReader(object):
             elem.getparent().remove(elem)
         fname = self.name.encode('ascii', 'replace')
         fname = re.sub(r'[\x08\x15\0]+', '', fname)
+        if not fname:
+            fname = 'dummy'
         htmlfile = os.path.join(output_dir,
             ascii_filename(fname) + '.html')
         try:
@@ -441,7 +446,7 @@ class MobiReader(object):
                 html.tostring(elem, encoding='utf-8') + '</package>'
         stream = cStringIO.StringIO(raw)
         opf = OPF(stream)
-        self.embedded_mi = MetaInformation(opf)
+        self.embedded_mi = opf.to_book_metadata()
         if guide is not None:
             for ref in guide.xpath('descendant::reference'):
                 if 'cover' in ref.get('type', '').lower():
@@ -564,6 +569,10 @@ class MobiReader(object):
                 for attr in self.IMAGE_ATTRS:
                     recindex = attrib.pop(attr, None) or recindex
                 if recindex is not None:
+                    try:
+                        recindex = '%05d'%int(recindex)
+                    except:
+                        pass
                     attrib['src'] = 'images/%s.jpg' % recindex
                 for attr in ('width', 'height'):
                     if attr in attrib:
