@@ -126,24 +126,29 @@ class OEBReader(object):
 
     def _metadata_from_opf(self, opf):
         from calibre.ebooks.metadata.opf2 import OPF
-        from calibre.ebooks.metadata import MetaInformation
         from calibre.ebooks.oeb.transforms.metadata import meta_info_to_oeb_metadata
         stream = cStringIO.StringIO(etree.tostring(opf))
-        mi = MetaInformation(OPF(stream))
+        mi = OPF(stream).to_book_metadata()
         if not mi.language:
             mi.language = get_lang().replace('_', '-')
         self.oeb.metadata.add('language', mi.language)
-        if not mi.title:
-            mi.title = self.oeb.translate(__('Unknown'))
-        if not mi.authors:
-            mi.authors = [self.oeb.translate(__('Unknown'))]
         if not mi.book_producer:
-            mi.book_producer = '%(a)s (%(v)s) [http://%(a)s.kovidgoyal.net]'%\
+            mi.book_producer = '%(a)s (%(v)s) [http://%(a)s-ebook.com]'%\
                 dict(a=__appname__, v=__version__)
         meta_info_to_oeb_metadata(mi, self.oeb.metadata, self.logger)
-        self.oeb.metadata.add('identifier', str(uuid.uuid4()), id='uuid_id',
-                                scheme='uuid')
+        m = self.oeb.metadata
+        m.add('identifier', str(uuid.uuid4()), id='uuid_id', scheme='uuid')
         self.oeb.uid = self.oeb.metadata.identifier[-1]
+        if not m.title:
+            m.add('title', self.oeb.translate(__('Unknown')))
+        has_aut = False
+        for x in m.creator:
+            if getattr(x, 'role', '').lower() in ('', 'aut'):
+                has_aut = True
+                break
+        if not has_aut:
+            m.add('creator', self.oeb.translate(__('Unknown')), role='aut')
+
 
     def _manifest_prune_invalid(self):
         '''
