@@ -5,17 +5,12 @@ __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import textwrap, cStringIO
+import textwrap
 from urllib import unquote
 
 from lxml import etree
-try:
-    from PIL import Image as PILImage
-    PILImage
-except ImportError:
-    import Image as PILImage
-
 from calibre import guess_type
+from calibre.utils.magick.draw import identify_data
 
 class CoverManager(object):
 
@@ -31,12 +26,14 @@ class CoverManager(object):
                 </style>
             </head>
             <body>
-                <svg version="1.1" xmlns="http://www.w3.org/2000/svg"
-                    xmlns:xlink="http://www.w3.org/1999/xlink"
-                    width="100%%" height="100%%" viewBox="__viewbox__"
-                    preserveAspectRatio="__ar__">
-                    <image width="__width__" height="__height__" xlink:href="%s"/>
-                </svg>
+                <div>
+                    <svg version="1.1" xmlns="http://www.w3.org/2000/svg"
+                        xmlns:xlink="http://www.w3.org/1999/xlink"
+                        width="100%%" height="100%%" viewBox="__viewbox__"
+                        preserveAspectRatio="__ar__">
+                        <image width="__width__" height="__height__" xlink:href="%s"/>
+                    </svg>
+                </div>
             </body>
         </html>
         ''')
@@ -99,7 +96,8 @@ class CoverManager(object):
         series_string = None
         if m.series and m.series_index:
             series_string = _('Book %s of %s')%(
-                    fmt_sidx(m.series_index[0], use_roman=True), m.series[0])
+                    fmt_sidx(m.series_index[0], use_roman=True),
+                    unicode(m.series[0]))
 
         try:
             from calibre.ebooks import calibre_cover
@@ -123,9 +121,7 @@ class CoverManager(object):
             if x.href == urlnormalize(href):
                 try:
                     raw = x.data
-                    f = cStringIO.StringIO(raw)
-                    im = PILImage.open(f)
-                    return im.size
+                    return identify_data(raw)[:2]
                 except:
                     self.log.exception('Failed to read image dimensions')
         return None, None
