@@ -1,8 +1,9 @@
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 """ The GUI """
-import os, sys
+import os, sys, Queue, threading
 from threading import RLock
+from urllib import unquote
 
 from PyQt4.Qt import QVariant, QFileInfo, QObject, SIGNAL, QBuffer, Qt, \
                          QByteArray, QTranslator, QCoreApplication, QThread, \
@@ -39,7 +40,7 @@ gprefs.defaults['action-layout-context-menu'] = (
         'Edit Metadata', 'Send To Device', 'Save To Disk',
         'Connect Share', 'Copy To Library', None,
         'Convert Books', 'View', 'Open Folder', 'Show Book Details',
-        'Similar Books', None, 'Remove Books',
+        'Similar Books', 'Tweak ePub', None, 'Remove Books',
         )
 
 gprefs.defaults['action-layout-context-menu-device'] = (
@@ -50,6 +51,7 @@ gprefs.defaults['action-layout-context-menu-device'] = (
 gprefs.defaults['show_splash_screen'] = True
 gprefs.defaults['toolbar_icon_size'] = 'medium'
 gprefs.defaults['toolbar_text'] = 'auto'
+gprefs.defaults['show_child_bar'] = False
 
 # }}}
 
@@ -221,7 +223,6 @@ class MessageBox(QMessageBox):
         if default_button is not None:
             self.setDefaultButton(default_button)
 
-
     def copy_to_clipboard(self):
         QApplication.clipboard().setText('%s: %s\n\n%s' %
                 (self.title, self.msg, self.det_msg))
@@ -253,7 +254,7 @@ def error_dialog(parent, title, msg, det_msg='', show=False,
     return d
 
 def question_dialog(parent, title, msg, det_msg='', show_copy_button=True,
-        buttons=QMessageBox.Yes|QMessageBox.No):
+        buttons=QMessageBox.Yes|QMessageBox.No, yes_button=QMessageBox.Yes):
     d = MessageBox(QMessageBox.Question, title, msg, buttons,
                     parent, det_msg)
     d.setIconPixmap(QPixmap(I('dialog_question.png')))
@@ -261,7 +262,7 @@ def question_dialog(parent, title, msg, det_msg='', show_copy_button=True,
     if not show_copy_button:
         d.cb.setVisible(False)
 
-    return d.exec_() == QMessageBox.Yes
+    return d.exec_() == yes_button
 
 def info_dialog(parent, title, msg, det_msg='', show=False):
     d = MessageBox(QMessageBox.Information, title, msg, QMessageBox.Ok,
@@ -295,6 +296,37 @@ class Dispatcher(QObject):
     def dispatch(self, args, kwargs):
         self.func(*args, **kwargs)
 
+class FunctionDispatcher(QObject):
+    '''
+    Convenience class to use Qt signals with arbitrary python functions.
+    By default, ensures that a function call always happens in the
+    thread this Dispatcher was created in.
+    '''
+    dispatch_signal = pyqtSignal(object, object, object)
+
+    def __init__(self, func, queued=True, parent=None):
+        QObject.__init__(self, parent)
+        self.func = func
+        typ = Qt.QueuedConnection
+        if not queued:
+            typ = Qt.AutoConnection if queued is None else Qt.DirectConnection
+        self.dispatch_signal.connect(self.dispatch, type=typ)
+        self.q = Queue.Queue()
+        self.lock = threading.Lock()
+
+    def __call__(self, *args, **kwargs):
+        with self.lock:
+            self.dispatch_signal.emit(self.q, args, kwargs)
+            res = self.q.get()
+        return res
+
+    def dispatch(self, q, args, kwargs):
+        try:
+            res = self.func(*args, **kwargs)
+        except:
+            res = None
+        q.put(res)
+
 class GetMetadata(QObject):
     '''
     Convenience class to ensure that metadata readers are used only in the
@@ -366,6 +398,7 @@ class FileIconProvider(QFileIconProvider):
              'fb2'     : 'fb2',
              'rtf'     : 'rtf',
              'odt'     : 'odt',
+             'snb'     : 'snb',
              }
 
     def __init__(self):
@@ -473,10 +506,15 @@ class FileDialog(QObject):
             fs = QFileDialog.getOpenFileNames(parent, title, initial_dir, ftext, "")
             for f in fs:
                 f = unicode(f)
+                if not f: continue
+                if not os.path.exists(f):
+                    # QFileDialog for some reason quotes spaces
+                    # on linux if there is more than one space in a row
+                    f = unquote(f)
                 if f and os.path.exists(f):
                     self.selected_files.append(f)
         else:
-            opts = QFileDialog.ShowDirsOnly if mode == QFileDialog.DirectoryOnly else QFileDialog.Option()
+            opts = QFileDialog.ShowDirsOnly if mode == QFileDialog.Directory else QFileDialog.Option()
             f = unicode(QFileDialog.getExistingDirectory(parent, title, initial_dir, opts))
             if os.path.exists(f):
                 self.selected_files.append(f)
@@ -496,7 +534,7 @@ class FileDialog(QObject):
 
 def choose_dir(window, name, title, default_dir='~'):
     fd = FileDialog(title=title, filters=[], add_all_files_filter=False,
-            parent=window, name=name, mode=QFileDialog.DirectoryOnly,
+            parent=window, name=name, mode=QFileDialog.Directory,
             default_dir=default_dir)
     dir = fd.get_files()
     if dir:
@@ -574,18 +612,6 @@ class Application(QApplication):
         self._file_open_paths = []
         self._file_open_lock = RLock()
 
-        if islinux:
-            self.setStyleSheet('''
-                    QToolTip {
-                        border: 2px solid black;
-                        padding: 5px;
-                        border-radius: 10px;
-                        opacity: 200;
-                        background-color: #e1e1ff;
-                        color: black;
-                    }
-            ''')
-
     def _send_file_open_events(self):
         with self._file_open_lock:
             if self._file_open_paths:
@@ -688,13 +714,13 @@ def build_forms(srcdir, info=None):
             dat = re.compile(r'QtGui.QApplication.translate\(.+?,\s+"(.+?)(?<!\\)",.+?\)', re.DOTALL).sub(r'_("\1")', dat)
             dat = dat.replace('_("MMM yyyy")', '"MMM yyyy"')
             dat = pat.sub(sub, dat)
+            dat = dat.replace('from QtWebKit.QWebView import QWebView',
+                    'from PyQt4 import QtWebKit\nfrom PyQt4.QtWebKit import QWebView')
 
             if form.endswith('viewer%smain.ui'%os.sep):
                 info('\t\tPromoting WebView')
                 dat = dat.replace('self.view = QtWebKit.QWebView(', 'self.view = DocumentView(')
                 dat = dat.replace('self.view = QWebView(', 'self.view = DocumentView(')
-                dat = dat.replace('from QtWebKit.QWebView import QWebView',
-                        'from PyQt4 import QtWebKit\nfrom PyQt4.QtWebKit import QWebView')
                 dat += '\n\nfrom calibre.gui2.viewer.documentview import DocumentView'
 
             open(compiled_form, 'wb').write(dat)
