@@ -25,6 +25,7 @@ class AddAction(InterfaceAction):
     action_spec = (_('Add books'), 'add_book.png',
             _('Add books to the calibre library/device from files on your computer')
             , _('A'))
+    action_type = 'current'
 
     def genesis(self):
         self._add_filesystem_book = self.Dispatcher(self.__add_filesystem_book)
@@ -165,6 +166,7 @@ class AddAction(InterfaceAction):
                         (_('Topaz books'), ['tpz','azw1']),
                         (_('Text books'), ['txt', 'rtf']),
                         (_('PDF Books'), ['pdf']),
+                        (_('SNB Books'), ['snb']),
                         (_('Comics'), ['cbz', 'cbr', 'cbc']),
                         (_('Archives'), ['zip', 'rar']),
                         ]
@@ -229,17 +231,22 @@ class AddAction(InterfaceAction):
         self._files_added(paths, names, infos, on_card=on_card)
         # set the in-library flags, and as a consequence send the library's
         # metadata for this book to the device. This sets the uuid to the
-        # correct value.
+        # correct value. Note that set_books_in_library might sync_booklists
         self.gui.set_books_in_library(booklists=[model.db], reset=True)
-        model.reset()
+        self.gui.refresh_ondevice()
 
-    def add_books_from_device(self, view):
-        rows = view.selectionModel().selectedRows()
-        if not rows or len(rows) == 0:
-            d = error_dialog(self.gui, _('Add to library'), _('No book selected'))
-            d.exec_()
-            return
-        paths = [p for p in view._model.paths(rows) if p is not None]
+    def add_books_from_device(self, view, paths=None):
+        backloading_err = self.gui.device_manager.device.BACKLOADING_ERROR_MESSAGE
+        if backloading_err is not None:
+            return error_dialog(self.gui, _('Add to library'), backloading_err,
+                    show=True)
+        if paths is None:
+            rows = view.selectionModel().selectedRows()
+            if not rows or len(rows) == 0:
+                d = error_dialog(self.gui, _('Add to library'), _('No book selected'))
+                d.exec_()
+                return
+            paths = [p for p in view.model().paths(rows) if p is not None]
         ve = self.gui.device_manager.device.VIRTUAL_BOOK_EXTENSIONS
         def ext(x):
             ans = os.path.splitext(x)[1]
@@ -260,7 +267,7 @@ class AddAction(InterfaceAction):
             return
         from calibre.gui2.add import Adder
         self.__adder_func = partial(self._add_from_device_adder, on_card=None,
-                                                    model=view._model)
+                                                    model=view.model())
         self._adder = Adder(self.gui, self.gui.library_view.model().db,
                 self.Dispatcher(self.__adder_func), spare_server=self.gui.spare_server)
         self._adder.add(paths)
