@@ -14,7 +14,7 @@ from calibre.gui2.convert.regex_builder_ui import Ui_RegexBuilder
 from calibre.gui2.convert.xexp_edit_ui import Ui_Form as Ui_Edit
 from calibre.gui2 import error_dialog, choose_files
 from calibre.ebooks.oeb.iterator import EbookIterator
-from calibre.ebooks.conversion.preprocess import convert_entities
+from calibre.ebooks.conversion.preprocess import HTMLPreProcessor
 from calibre.gui2.dialogs.choose_format import ChooseFormatDialog
 
 class RegexBuilder(QDialog, Ui_RegexBuilder):
@@ -28,9 +28,10 @@ class RegexBuilder(QDialog, Ui_RegexBuilder):
 
         if not db or not book_id:
             self.button_box.addButton(QDialogButtonBox.Open)
-        else:
-            self.select_format(db, book_id)
-
+        elif not self.select_format(db, book_id):
+            self.cancelled = True
+            return
+        self.cancelled = False
         self.connect(self.button_box, SIGNAL('clicked(QAbstractButton*)'), self.button_clicked)
         self.connect(self.regex, SIGNAL('textChanged(QString)'), self.regex_valid)
         self.connect(self.test, SIGNAL('clicked()'), self.do_test)
@@ -79,19 +80,21 @@ class RegexBuilder(QDialog, Ui_RegexBuilder):
                 format = d.format()
 
         if not format:
-            error_dialog(self, _('No formats available'), _('Cannot build regex using the GUI builder without a book.'))
-            QDialog.reject()
-        else:
-            self.open_book(db.format_abspath(book_id, format, index_is_id=True))
+            error_dialog(self, _('No formats available'),
+                         _('Cannot build regex using the GUI builder without a book.'),
+                         show=True)
+            return False
+        self.open_book(db.format_abspath(book_id, format, index_is_id=True))
+        return True
 
     def open_book(self, pathtoebook):
         self.iterator = EbookIterator(pathtoebook)
         self.iterator.__enter__(only_input_plugin=True)
         text = [u'']
-        ent_pat = re.compile(r'&(\S+?);')
+        preprocessor = HTMLPreProcessor(None, False)
         for path in self.iterator.spine:
             html = open(path, 'rb').read().decode('utf-8', 'replace')
-            html = ent_pat.sub(convert_entities, html)
+            html = preprocessor(html, get_preprocess_html=True)
             text.append(html)
         self.preview.setPlainText('\n---\n'.join(text))
 
@@ -117,6 +120,8 @@ class RegexEdit(QWidget, Ui_Edit):
 
     def builder(self):
         bld = RegexBuilder(self.db, self.book_id, self.edit.text(), self)
+        if bld.cancelled:
+            return
         if bld.exec_() == bld.Accepted:
             self.edit.setText(bld.regex.text())
 
