@@ -299,7 +299,9 @@ class Series(Base):
         val, s_index = self.gui_val
         val = self.normalize_ui_val(val)
         if val != self.initial_val or s_index != self.initial_index:
-            if s_index == 0.0:
+            if val == '':
+                val = s_index = None
+            elif s_index == 0.0:
                 if tweaks['series_index_auto_increment'] == 'next':
                     s_index = self.db.get_next_cc_series_num_for(val,
                                                              num=self.col_id)
@@ -348,7 +350,11 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
     ans = []
     column = row = comments_row = 0
     for col in cols:
+        if not x[col]['editable']:
+            continue
         dt = x[col]['datatype']
+        if dt == 'composite':
+            continue
         if dt == 'comments':
             continue
         w = widget_factory(dt, col)
@@ -423,7 +429,38 @@ class BulkBase(Base):
             self.db.set_custom_bulk(book_ids, val, num=self.col_id, notify=notify)
 
 class BulkBool(BulkBase, Bool):
-    pass
+
+    def get_initial_value(self, book_ids):
+        value = None
+        for book_id in book_ids:
+            val = self.db.get_custom(book_id, num=self.col_id, index_is_id=True)
+            if tweaks['bool_custom_columns_are_tristate'] == 'no' and val is None:
+                val = False
+            if value is not None and value != val:
+                return None
+            value = val
+        return value
+
+    def setup_ui(self, parent):
+        self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent),
+                QComboBox(parent)]
+        w = self.widgets[1]
+        items = [_('Yes'), _('No'), _('Undefined')]
+        icons = [I('ok.png'), I('list_remove.png'), I('blank.png')]
+        for icon, text in zip(icons, items):
+            w.addItem(QIcon(icon), text)
+
+    def setter(self, val):
+        val = {None: 2, False: 1, True: 0}[val]
+        self.widgets[1].setCurrentIndex(val)
+
+    def commit(self, book_ids, notify=False):
+        val = self.gui_val
+        val = self.normalize_ui_val(val)
+        if val != self.initial_val:
+            if tweaks['bool_custom_columns_are_tristate'] == 'no' and val is None:
+                val = False
+            self.db.set_custom_bulk(book_ids, val, num=self.col_id, notify=notify)
 
 class BulkInt(BulkBase, Int):
     pass
@@ -448,9 +485,25 @@ class BulkSeries(BulkBase):
         self.name_widget = w
         self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent), w]
 
-        self.widgets.append(QLabel(_('Automatically number books in this series'), parent))
-        self.idx_widget=QCheckBox(parent)
-        self.widgets.append(self.idx_widget)
+        self.widgets.append(QLabel('', parent))
+        w = QWidget(parent)
+        layout = QHBoxLayout(w)
+        layout.setContentsMargins(0, 0, 0, 0)
+        self.remove_series = QCheckBox(parent)
+        self.remove_series.setText(_('Remove series'))
+        layout.addWidget(self.remove_series)
+        self.idx_widget = QCheckBox(parent)
+        self.idx_widget.setText(_('Automatically number books'))
+        layout.addWidget(self.idx_widget)
+        self.force_number = QCheckBox(parent)
+        self.force_number.setText(_('Force numbers to start with '))
+        layout.addWidget(self.force_number)
+        self.series_start_number = QSpinBox(parent)
+        self.series_start_number.setMinimum(1)
+        self.series_start_number.setProperty("value", 1)
+        layout.addWidget(self.series_start_number)
+        layout.addItem(QSpacerItem(20, 10, QSizePolicy.Expanding, QSizePolicy.Minimum))
+        self.widgets.append(w)
 
     def initialize(self, book_id):
         self.idx_widget.setChecked(False)
@@ -461,17 +514,26 @@ class BulkSeries(BulkBase):
     def getter(self):
         n = unicode(self.name_widget.currentText()).strip()
         i = self.idx_widget.checkState()
-        return n, i
+        f = self.force_number.checkState()
+        s = self.series_start_number.value()
+        r = self.remove_series.checkState()
+        return n, i, f, s, r
 
     def commit(self, book_ids, notify=False):
-        val, update_indices = self.gui_val
-        val = self.normalize_ui_val(val)
-        if val != '':
+        val, update_indices, force_start, at_value, clear = self.gui_val
+        val = None if clear else self.normalize_ui_val(val)
+        if clear or val != '':
             extras = []
             next_index = self.db.get_next_cc_series_num_for(val, num=self.col_id)
             for book_id in book_ids:
+                if clear:
+                    extras.append(None)
+                    continue
                 if update_indices:
-                    if tweaks['series_index_auto_increment'] == 'next':
+                    if force_start:
+                        s_index = at_value
+                        at_value += 1
+                    elif tweaks['series_index_auto_increment'] == 'next':
                         s_index = next_index
                         next_index += 1
                     else:
@@ -479,6 +541,8 @@ class BulkSeries(BulkBase):
                 else:
                     s_index = self.db.get_custom_extra(book_id, num=self.col_id,
                                                        index_is_id=True)
+                    if s_index is None:
+                        s_index = 1.0
                 extras.append(s_index)
             self.db.set_custom_bulk(book_ids, val, extras=extras,
                                    num=self.col_id, notify=notify)
