@@ -20,10 +20,9 @@ from calibre.gui2.dialogs.choose_format import ChooseFormatDialog
 from calibre.utils.ipc.job import BaseJob
 from calibre.devices.scanner import DeviceScanner
 from calibre.gui2 import config, error_dialog, Dispatcher, dynamic, \
-                        warning_dialog, \
-                        question_dialog, info_dialog, choose_dir
+        warning_dialog, info_dialog, choose_dir
 from calibre.ebooks.metadata import authors_to_string
-from calibre import preferred_encoding, prints
+from calibre import preferred_encoding, prints, force_unicode
 from calibre.utils.filenames import ascii_filename
 from calibre.devices.errors import FreeSpaceError
 from calibre.utils.smtp import compose_mail, sendmail, extract_email_address, \
@@ -34,6 +33,8 @@ from calibre.ebooks.metadata.meta import set_metadata
 from calibre.constants import DEBUG
 from calibre.utils.config import prefs, tweaks
 from calibre.utils.magick.draw import thumbnail
+from calibre.library.save_to_disk import plugboard_any_device_value, \
+                                         plugboard_any_format_value
 # }}}
 
 class DeviceJob(BaseJob): # {{{
@@ -102,6 +103,28 @@ class DeviceJob(BaseJob): # {{{
 
     # }}}
 
+def find_plugboard(device_name, format, plugboards):
+    cpb = None
+    if format in plugboards:
+        cpb = plugboards[format]
+    elif plugboard_any_format_value in plugboards:
+        cpb = plugboards[plugboard_any_format_value]
+    if cpb is not None:
+        if device_name in cpb:
+            cpb = cpb[device_name]
+        elif plugboard_any_device_value in cpb:
+            cpb = cpb[plugboard_any_device_value]
+        else:
+            cpb = None
+    if DEBUG:
+        prints('Device using plugboard', format, device_name, cpb)
+    return cpb
+
+def device_name_for_plugboards(device_class):
+    if hasattr(device_class, 'DEVICE_PLUGBOARD_NAME'):
+        return device_class.DEVICE_PLUGBOARD_NAME
+    return device_class.__class__.__name__
+
 class DeviceManager(Thread): # {{{
 
     def __init__(self, connected_slot, job_manager, open_feedback_slot, sleep_time=2):
@@ -308,7 +331,10 @@ class DeviceManager(Thread): # {{{
         self.device.sync_booklists(booklists, end_session=False)
         return self.device.card_prefix(end_session=False), self.device.free_space()
 
-    def sync_booklists(self, done, booklists):
+    def sync_booklists(self, done, booklists, plugboards):
+        if hasattr(self.connected_device, 'set_plugboards') and \
+                callable(self.connected_device.set_plugboards):
+            self.connected_device.set_plugboards(plugboards, find_plugboard)
         return self.create_job(self._sync_booklists, done, args=[booklists],
                         description=_('Send metadata to device'))
 
@@ -317,19 +343,30 @@ class DeviceManager(Thread): # {{{
                                args=[booklist, on_card],
                         description=_('Send collections to device'))
 
-    def _upload_books(self, files, names, on_card=None, metadata=None):
+    def _upload_books(self, files, names, on_card=None, metadata=None, plugboards=None):
         '''Upload books to device: '''
+        if hasattr(self.connected_device, 'set_plugboards') and \
+                callable(self.connected_device.set_plugboards):
+            self.connected_device.set_plugboards(plugboards, find_plugboard)
         if metadata and files and len(metadata) == len(files):
             for f, mi in zip(files, metadata):
                 if isinstance(f, unicode):
                     ext = f.rpartition('.')[-1].lower()
+                    cpb = find_plugboard(
+                            device_name_for_plugboards(self.connected_device),
+                            ext, plugboards)
                     if ext:
                         try:
                             if DEBUG:
                                 prints('Setting metadata in:', mi.title, 'at:',
                                         f, file=sys.__stdout__)
                             with open(f, 'r+b') as stream:
-                                set_metadata(stream, mi, stream_type=ext)
+                                if cpb:
+                                    newmi = mi.deepcopy_metadata()
+                                    newmi.template_to_attribute(mi, cpb)
+                                else:
+                                    newmi = mi
+                                set_metadata(stream, newmi, stream_type=ext)
                         except:
                             if DEBUG:
                                 prints(traceback.format_exc(), file=sys.__stdout__)
@@ -338,12 +375,12 @@ class DeviceManager(Thread): # {{{
                                         metadata=metadata, end_session=False)
 
     def upload_books(self, done, files, names, on_card=None, titles=None,
-                     metadata=None):
+                     metadata=None, plugboards=None):
         desc = _('Upload %d books to device')%len(names)
         if titles:
             desc += u':' + u', '.join(titles)
         return self.create_job(self._upload_books, done, args=[files, names],
-                kwargs={'on_card':on_card,'metadata':metadata}, description=desc)
+                kwargs={'on_card':on_card,'metadata':metadata,'plugboards':plugboards}, description=desc)
 
     def add_books_to_metadata(self, locations, metadata, booklists):
         self.device.add_books_to_metadata(locations, metadata, booklists)
@@ -446,17 +483,22 @@ class DeviceMenu(QMenu): # {{{
                     _('Storage Card B')),
         ]
 
+        later_menus = []
 
         for menu in (self, self.set_default_menu):
             for actions, desc in (
                     (basic_actions, ''),
+                    (specific_actions, _('Send specific format to')),
                     (delete_actions, _('Send and delete from library')),
-                    (specific_actions, _('Send specific format'))
                     ):
                 mdest = menu
                 if actions is not basic_actions:
-                    mdest = menu.addMenu(desc)
+                    mdest = QMenu(desc)
                     self._memory.append(mdest)
+                    later_menus.append(mdest)
+                    if menu is self.set_default_menu:
+                        menu.addMenu(mdest)
+                        menu.addSeparator()
 
                 for dest, delete, specific, icon, text in actions:
                     action = DeviceAction(dest, delete, specific, icon, text, self)
@@ -469,7 +511,7 @@ class DeviceMenu(QMenu): # {{{
                         action.a_s.connect(self.action_triggered)
                         self.actions.append(action)
                     mdest.addAction(action)
-                if actions is not specific_actions:
+                if actions is basic_actions:
                     menu.addSeparator()
 
         da = config['default_send_to_device_action']
@@ -487,14 +529,21 @@ class DeviceMenu(QMenu): # {{{
         self.group.triggered.connect(self.change_default_action)
         self.addSeparator()
 
+        self.addMenu(later_menus[0])
+        self.addSeparator()
+
         mitem = self.addAction(QIcon(I('eject.png')), _('Eject device'))
         mitem.setEnabled(False)
         mitem.triggered.connect(lambda x : self.disconnect_mounted_device.emit())
         self.disconnect_mounted_device_action = mitem
-
         self.addSeparator()
+
         self.addMenu(self.set_default_menu)
         self.addSeparator()
+
+        self.addMenu(later_menus[1])
+        self.addSeparator()
+
         annot = self.addAction(_('Fetch annotations (experimental)'))
         annot.setEnabled(False)
         annot.triggered.connect(lambda x :
@@ -615,6 +664,16 @@ class DeviceMixin(object): # {{{
         if tweaks['auto_connect_to_folder']:
             self.connect_to_folder_named(tweaks['auto_connect_to_folder'])
 
+    def auto_convert_question(self, msg, autos):
+        autos = u'\n'.join(map(unicode, map(force_unicode, autos)))
+        return self.ask_a_yes_no_question(
+                _('No suitable formats'), msg,
+                buttons=QMessageBox.Yes|QMessageBox.Cancel,
+                ans_when_user_unavailable=True,
+                det_msg=autos,
+                show_copy_button=False
+        )
+
     def set_default_thumbnail(self, height):
         img = I('book.png', data=True)
         self.default_thumbnail = thumbnail(img, height, height)
@@ -627,12 +686,11 @@ class DeviceMixin(object): # {{{
     def connect_to_folder(self):
         dir = choose_dir(self, 'Select Device Folder',
                              _('Select folder to open as device'))
-        kls = FOLDER_DEVICE
-        self.device_manager.mount_device(kls=kls, kind='folder', path=dir)
+        if dir is not None:
+            self.device_manager.mount_device(kls=FOLDER_DEVICE, kind='folder', path=dir)
 
     def connect_to_itunes(self):
-        kls = ITUNES_ASYNC
-        self.device_manager.mount_device(kls=kls, kind='itunes', path=None)
+        self.device_manager.mount_device(kls=ITUNES_ASYNC, kind='itunes', path=None)
 
     # disconnect from both folder and itunes devices
     def disconnect_mounted_device(self):
@@ -708,6 +766,10 @@ class DeviceMixin(object): # {{{
         '''
         Called when a device is connected to the computer.
         '''
+        # This can happen as this function is called in a queued connection and
+        # the user could have yanked the device in the meantime
+        if connected and not self.device_manager.is_device_connected:
+            connected = False
         self.set_device_menu_items_state(connected)
         if connected:
             self.device_manager.get_device_information(\
@@ -718,14 +780,16 @@ class DeviceMixin(object): # {{{
                 self.device_manager.device.__class__.get_gui_name()+\
                         _(' detected.'), 3000)
             self.device_connected = device_kind
-            self.refresh_ondevice_info (device_connected = True, reset_only = True)
+            self.library_view.set_device_connected(self.device_connected)
+            self.refresh_ondevice (reset_only = True)
         else:
             self.device_connected = None
             self.status_bar.device_disconnected()
             if self.current_view() != self.library_view:
                 self.book_details.reset_info()
             self.location_manager.update_devices()
-            self.refresh_ondevice_info(device_connected=False)
+            self.library_view.set_device_connected(self.device_connected)
+            self.refresh_ondevice()
 
     def info_read(self, job):
         '''
@@ -746,19 +810,26 @@ class DeviceMixin(object): # {{{
         if job.failed:
             self.device_job_exception(job)
             return
+        # set_books_in_library might schedule a sync_booklists job
         self.set_books_in_library(job.result, reset=True)
         mainlist, cardalist, cardblist = job.result
         self.memory_view.set_database(mainlist)
-        self.memory_view.set_editable(self.device_manager.device.CAN_SET_METADATA)
+        self.memory_view.set_editable(self.device_manager.device.CAN_SET_METADATA,
+                                      self.device_manager.device.BACKLOADING_ERROR_MESSAGE
+                                      is None)
         self.card_a_view.set_database(cardalist)
-        self.card_a_view.set_editable(self.device_manager.device.CAN_SET_METADATA)
+        self.card_a_view.set_editable(self.device_manager.device.CAN_SET_METADATA,
+                                      self.device_manager.device.BACKLOADING_ERROR_MESSAGE
+                                      is None)
         self.card_b_view.set_database(cardblist)
-        self.card_b_view.set_editable(self.device_manager.device.CAN_SET_METADATA)
+        self.card_b_view.set_editable(self.device_manager.device.CAN_SET_METADATA,
+                                      self.device_manager.device.BACKLOADING_ERROR_MESSAGE
+                                      is None)
         self.sync_news()
         self.sync_catalogs()
-        self.refresh_ondevice_info(device_connected = True)
+        self.refresh_ondevice()
 
-    def refresh_ondevice_info(self, device_connected, reset_only = False):
+    def refresh_ondevice(self, reset_only = False):
         '''
         Force the library view to refresh, taking into consideration new
         device books information
@@ -766,7 +837,7 @@ class DeviceMixin(object): # {{{
         self.book_on_device(None, reset=True)
         if reset_only:
             return
-        self.library_view.set_device_connected(device_connected)
+        self.library_view.model().refresh_ondevice()
 
     # }}}
 
@@ -790,15 +861,16 @@ class DeviceMixin(object): # {{{
             self.device_manager.remove_books_from_metadata(paths,
                     self.booklists())
             model.paths_deleted(paths)
-            self.upload_booklists()
         # Force recomputation the library's ondevice info. We need to call
         # set_books_in_library even though books were not added because
-        # the deleted book might have been an exact match.
-        self.set_books_in_library(self.booklists(), reset=True)
-        self.book_on_device(None, None, reset=True)
+        # the deleted book might have been an exact match. Upload the booklists
+        # if set_books_in_library did not.
+        if not self.set_books_in_library(self.booklists(), reset=True):
+            self.upload_booklists()
+        self.book_on_device(None, reset=True)
         # We need to reset the ondevice flags in the library. Use a big hammer,
         # so we don't need to worry about whether some succeeded or not.
-        self.refresh_ondevice_info(device_connected=True, reset_only=False)
+        self.refresh_ondevice(reset_only=False)
 
     def dispatch_sync_event(self, dest, delete, specific):
         rows = self.library_view.selectionModel().selectedRows()
@@ -915,11 +987,9 @@ class DeviceMixin(object): # {{{
                 bad += auto
             else:
                 autos = [self.library_view.model().db.title(id, index_is_id=True) for id in auto]
-                autos = '\n'.join('%s'%i for i in autos)
-                if question_dialog(self, _('No suitable formats'),
+                if self.auto_convert_question(
                     _('Auto convert the following books before sending via '
-                        'email?'), det_msg=autos,
-                    buttons=QMessageBox.Yes|QMessageBox.Cancel):
+                        'email?'), autos):
                     self.iactions['Convert Books'].auto_convert_mail(to, fmts, delete_from_library, auto, format)
 
         if bad:
@@ -934,12 +1004,12 @@ class DeviceMixin(object): # {{{
         for jobname, exception, tb in results:
             title = jobname.partition(':')[-1]
             if exception is not None:
-                errors.append([title, exception, tb])
+                errors.append(list(map(force_unicode, [title, exception, tb])))
             else:
                 good.append(title)
         if errors:
-            errors = '\n'.join([
-                    '%s\n\n%s\n%s\n' %
+            errors = u'\n'.join([
+                    u'%s\n\n%s\n%s\n' %
                     (title, e, tb) for \
                             title, e, tb in errors
                     ])
@@ -951,7 +1021,13 @@ class DeviceMixin(object): # {{{
             self.status_bar.show_message(_('Sent by email:') + ', '.join(good),
                     5000)
             if remove:
-                self.library_view.model().delete_books_by_id(remove)
+                try:
+                    self.library_view.model().delete_books_by_id(remove)
+                except:
+                    # Probably the user deleted the files, in any case, failing
+                    # to delete the book is not catastrophic
+                    traceback.print_exc()
+
 
     def cover_to_thumbnail(self, data):
         ht = self.device_manager.device.THUMBNAIL_HEIGHT \
@@ -978,7 +1054,7 @@ class DeviceMixin(object): # {{{
             to_s = [account]
             subjects = [_('News:')+' '+mi.title]
             texts    = [_('Attached is the')+' '+mi.title]
-            attachment_names = [mi.title+os.path.splitext(attachment)[1]]
+            attachment_names = [ascii_filename(mi.title)+os.path.splitext(attachment)[1]]
             attachments = [attachment]
             jobnames = ['%s:%s'%(id, mi.title)]
             remove = [id] if config['delete_news_from_library_on_upload']\
@@ -1016,11 +1092,9 @@ class DeviceMixin(object): # {{{
                         break
                 if format is not None:
                     autos = [self.library_view.model().db.title(id, index_is_id=True) for id in auto]
-                    autos = '\n'.join('%s'%i for i in autos)
-                    if question_dialog(self, _('No suitable formats'),
+                    if self.auto_convert_question(
                         _('Auto convert the following books before uploading to '
-                            'the device?'), det_msg=autos,
-                        buttons=QMessageBox.Yes|QMessageBox.Cancel):
+                            'the device?'), autos):
                         self.iactions['Convert Books'].auto_convert_catalogs(auto, format)
             files = [f for f in files if f is not None]
             if not files:
@@ -1051,12 +1125,35 @@ class DeviceMixin(object): # {{{
                 self.status_bar.show_message(_('Sending catalogs to device.'), 5000)
 
 
+    @dynamic_property
+    def news_to_be_synced(self):
+        doc = 'Set of ids to be sent to device'
+        def fget(self):
+            ans = []
+            try:
+                ans = self.library_view.model().db.prefs.get('news_to_be_synced',
+                        [])
+            except:
+                import traceback
+                traceback.print_exc()
+            return set(ans)
+
+        def fset(self, ids):
+            try:
+                self.library_view.model().db.prefs.set('news_to_be_synced',
+                        list(ids))
+            except:
+                import traceback
+                traceback.print_exc()
+
+        return property(fget=fget, fset=fset, doc=doc)
+
 
     def sync_news(self, send_ids=None, do_auto_convert=True):
         if self.device_connected:
             del_on_upload = config['delete_news_from_library_on_upload']
             settings = self.device_manager.device.settings()
-            ids = list(dynamic.get('news_to_be_synced', set([]))) if send_ids is None else send_ids
+            ids = list(self.news_to_be_synced) if send_ids is None else send_ids
             ids = [id for id in ids if self.library_view.model().db.has_id(id)]
             files, _auto_ids = self.library_view.model().get_preferred_formats_from_ids(
                                 ids, settings.format_map,
@@ -1078,17 +1175,15 @@ class DeviceMixin(object): # {{{
                         break
                 if format is not None:
                     autos = [self.library_view.model().db.title(id, index_is_id=True) for id in auto]
-                    autos = '\n'.join('%s'%i for i in autos)
-                    if question_dialog(self, _('No suitable formats'),
+                    if self.auto_convert_question(
                         _('Auto convert the following books before uploading to '
-                            'the device?'), det_msg=autos,
-                        buttons=QMessageBox.Yes|QMessageBox.Cancel):
+                            'the device?'), autos):
                         self.iactions['Convert Books'].auto_convert_news(auto, format)
             files = [f for f in files if f is not None]
             for f in files:
                 f.deleted_after_upload = del_on_upload
             if not files:
-                dynamic.set('news_to_be_synced', set([]))
+                self.news_to_be_synced = set([])
                 return
             metadata = self.library_view.model().metadata_for(ids)
             names = []
@@ -1102,7 +1197,7 @@ class DeviceMixin(object): # {{{
                 if mi.cover and os.access(mi.cover, os.R_OK):
                     mi.thumbnail = self.cover_to_thumbnail(open(mi.cover,
                         'rb').read())
-            dynamic.set('news_to_be_synced', set([]))
+            self.news_to_be_synced = set([])
             if config['upload_news_to_device'] and files:
                 remove = ids if del_on_upload else []
                 space = { self.location_manager.free[0] : None,
@@ -1197,11 +1292,9 @@ class DeviceMixin(object): # {{{
                 bad += auto
             else:
                 autos = [self.library_view.model().db.title(id, index_is_id=True) for id in auto]
-                autos = '\n'.join('%s'%i for i in autos)
-                if question_dialog(self, _('No suitable formats'),
+                if self.auto_convert_question(
                     _('Auto convert the following books before uploading to '
-                        'the device?'), det_msg=autos,
-                    buttons=QMessageBox.Yes|QMessageBox.Cancel):
+                        'the device?'), autos):
                     self.iactions['Convert Books'].auto_convert(auto, on_card, format)
 
         if bad:
@@ -1217,8 +1310,9 @@ class DeviceMixin(object): # {{{
         '''
         Upload metadata to device.
         '''
+        plugboards = self.library_view.model().db.prefs.get('plugboards', {})
         self.device_manager.sync_booklists(Dispatcher(self.metadata_synced),
-                                           self.booklists())
+                                           self.booklists(), plugboards)
 
     def metadata_synced(self, job):
         '''
@@ -1231,7 +1325,7 @@ class DeviceMixin(object): # {{{
         self.location_manager.update_devices(cp, fs,
                 self.device_manager.device.icon)
         # reset the views so that up-to-date info is shown. These need to be
-        # here because the sony driver updates collections in sync_booklists
+        # here because some drivers update collections in sync_booklists
         self.memory_view.reset()
         self.card_a_view.reset()
         self.card_b_view.reset()
@@ -1250,10 +1344,11 @@ class DeviceMixin(object): # {{{
         :param files: List of either paths to files or file like objects
         '''
         titles = [i.title for i in metadata]
+        plugboards = self.library_view.model().db.prefs.get('plugboards', {})
         job = self.device_manager.upload_books(
                 Dispatcher(self.books_uploaded),
                 files, names, on_card=on_card,
-                metadata=metadata, titles=titles
+                metadata=metadata, titles=titles, plugboards=plugboards
               )
         self.upload_memory[job] = (metadata, on_card, memory, files)
 
@@ -1281,8 +1376,6 @@ class DeviceMixin(object): # {{{
         self.device_manager.add_books_to_metadata(job.result,
                 metadata, self.booklists())
 
-        self.upload_booklists()
-
         books_to_be_deleted = []
         if memory and memory[1]:
             books_to_be_deleted = memory[1]
@@ -1292,18 +1385,22 @@ class DeviceMixin(object): # {{{
         # book already there with a different book. This happens frequently in
         # news. When this happens, the book match indication will be wrong
         # because the UUID changed. Force both the device and the library view
-        # to refresh the flags.
-        self.set_books_in_library(self.booklists(), reset=True)
-        self.book_on_device(None, reset=True)
-        self.refresh_ondevice_info(device_connected = True)
+        # to refresh the flags. Set_books_in_library could upload the booklists.
+        # If it does not, then do it here.
+        if not self.set_books_in_library(self.booklists(), reset=True):
+            self.upload_booklists()
+        with self.library_view.preserve_selected_books:
+            self.book_on_device(None, reset=True)
+            self.refresh_ondevice()
 
-        view = self.card_a_view if on_card == 'carda' else self.card_b_view if on_card == 'cardb' else self.memory_view
+        view = self.card_a_view if on_card == 'carda' else \
+            self.card_b_view if on_card == 'cardb' else self.memory_view
         view.model().resort(reset=False)
         view.model().research()
         for f in files:
             getattr(f, 'close', lambda : True)()
 
-    def book_on_device(self, id, format=None, reset=False):
+    def book_on_device(self, id, reset=False):
         '''
         Return an indication of whether the given book represented by its db id
         is on the currently connected device. It returns a 5 element list. The
@@ -1332,8 +1429,6 @@ class DeviceMixin(object): # {{{
                 self.book_db_id_cache.append(set())
                 for book in l:
                     db_id = getattr(book, 'application_id', None)
-                    if db_id is None:
-                        db_id = book.db_id
                     if db_id is not None:
                         # increment the count of books on the device with this
                         # db_id.
@@ -1367,15 +1462,16 @@ class DeviceMixin(object): # {{{
 
         # Force a reset if the caches are not initialized
         if reset or not hasattr(self, 'db_book_title_cache'):
+            # Build a cache (map) of the library, so the search isn't On**2
+            self.db_book_title_cache = {}
+            self.db_book_uuid_cache = {}
             # It might be possible to get here without having initialized the
             # library view. In this case, simply give up
             try:
                 db = self.library_view.model().db
             except:
-                return
-            # Build a cache (map) of the library, so the search isn't On**2
-            self.db_book_title_cache = {}
-            self.db_book_uuid_cache = {}
+                return False
+
             for id in db.data.iterallids():
                 mi = db.get_metadata(id, index_is_id=True)
                 title = clean_string(mi.title)
@@ -1409,7 +1505,7 @@ class DeviceMixin(object): # {{{
                     if update_metadata:
                         book.smart_update(self.db_book_uuid_cache[book.uuid],
                                           replace_metadata=True)
-                    book.in_library = True
+                    book.in_library = 'UUID'
                     # ensure that the correct application_id is set
                     book.application_id = \
                         self.db_book_uuid_cache[book.uuid].application_id
@@ -1422,21 +1518,21 @@ class DeviceMixin(object): # {{{
                     # will match if any of the db_id, author, or author_sort
                     # also match.
                     if getattr(book, 'application_id', None) in d['db_ids']:
-                        book.in_library = True
                         # app_id already matches a db_id. No need to set it.
                         if update_metadata:
                             book.smart_update(d['db_ids'][book.application_id],
                                               replace_metadata=True)
+                        book.in_library = 'APP_ID'
                         continue
                     # Sonys know their db_id independent of the application_id
                     # in the metadata cache. Check that as well.
                     if getattr(book, 'db_id', None) in d['db_ids']:
-                        book.in_library = True
-                        book.application_id = \
-                                    d['db_ids'][book.db_id].application_id
                         if update_metadata:
                             book.smart_update(d['db_ids'][book.db_id],
                                               replace_metadata=True)
+                        book.in_library = 'DB_ID'
+                        book.application_id = \
+                                    d['db_ids'][book.db_id].application_id
                         continue
                     # We now know that the application_id is not right. Set it
                     # to None to prevent book_on_device from accidentally
@@ -1448,29 +1544,34 @@ class DeviceMixin(object): # {{{
                         # either can appear as the author
                         book_authors = clean_string(authors_to_string(book.authors))
                         if book_authors in d['authors']:
-                            book.in_library = True
-                            book.application_id = \
-                                    d['authors'][book_authors].application_id
                             if update_metadata:
                                 book.smart_update(d['authors'][book_authors],
                                                   replace_metadata=True)
-                        elif book_authors in d['author_sort']:
-                            book.in_library = True
+                            book.in_library = 'AUTHOR'
                             book.application_id = \
-                                d['author_sort'][book_authors].application_id
+                                    d['authors'][book_authors].application_id
+                        elif book_authors in d['author_sort']:
                             if update_metadata:
                                 book.smart_update(d['author_sort'][book_authors],
                                                   replace_metadata=True)
+                            book.in_library = 'AUTH_SORT'
+                            book.application_id = \
+                                d['author_sort'][book_authors].application_id
                 else:
                     # Book definitely not matched. Clear its application ID
                     book.application_id = None
                 # Set author_sort if it isn't already
                 asort = getattr(book, 'author_sort', None)
                 if not asort and book.authors:
-                    book.author_sort = self.library_view.model().db.author_sort_from_authors(book.authors)
+                    book.author_sort = self.library_view.model().db.\
+                                author_sort_from_authors(book.authors)
 
         if update_metadata:
             if self.device_manager.is_device_connected:
-                self.device_manager.sync_booklists(None, booklists)
+                plugboards = self.library_view.model().db.prefs.get('plugboards', {})
+                self.device_manager.sync_booklists(
+                                    Dispatcher(self.metadata_synced), booklists,
+                                    plugboards)
+        return update_metadata
     # }}}
 
