@@ -6,6 +6,7 @@ __license__   = 'GPL v3'
 from PyQt4.Qt import Qt, QDialog, QTableWidgetItem, QAbstractItemView
 
 from calibre.ebooks.metadata import author_to_author_sort
+from calibre.gui2 import error_dialog
 from calibre.gui2.dialogs.edit_authors_dialog_ui import Ui_EditAuthorsDialog
 
 class tableItem(QTableWidgetItem):
@@ -109,6 +110,12 @@ class EditAuthorsDialog(QDialog, Ui_EditAuthorsDialog):
         if col == 0:
             item = self.table.item(row, 0)
             aut  = unicode(item.text()).strip()
+            amper = aut.find('&')
+            if amper >= 0:
+                error_dialog(self.parent(), _('Invalid author name'),
+                        _('Author names cannot contain & characters.')).exec_()
+                aut = aut.replace('&', '%')
+                self.table.item(row, 0).setText(aut)
             c = self.table.item(row, 1)
             c.setText(author_to_author_sort(aut))
             item = c
