@@ -6,17 +6,14 @@ The dialog used to edit meta information for a book as well as
 add/remove formats
 '''
 
-import os
-import re
-import time
-import traceback
+import os, re, time, traceback, textwrap
 
 from PyQt4.Qt import SIGNAL, QObject, Qt, QTimer, QThread, QDate, \
-    QPixmap, QListWidgetItem, QDialog, pyqtSignal
+    QPixmap, QListWidgetItem, QDialog, pyqtSignal, QMessageBox
 
 from calibre.gui2 import error_dialog, file_icon_provider, dynamic, \
                            choose_files, choose_images, ResizableDialog, \
-                           warning_dialog
+                           warning_dialog, question_dialog
 from calibre.gui2.dialogs.metadata_single_ui import Ui_MetadataSingleDialog
 from calibre.gui2.dialogs.fetch_metadata import FetchMetadata
 from calibre.gui2.dialogs.tag_editor import TagEditor
@@ -303,6 +300,24 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         self.cpixmap = pix
         self.cover_data = cdata
 
+    def trim_cover(self, *args):
+        from calibre.utils.magick import Image
+        cdata = self.cover_data
+        if not cdata:
+            return
+        im = Image()
+        im.load(cdata)
+        im.trim(10)
+        cdata = im.export('png')
+        pix = QPixmap()
+        pix.loadFromData(cdata)
+        self.cover.setPixmap(pix)
+        self.cover_changed = True
+        self.cpixmap = pix
+        self.cover_data = cdata
+
+
+
     def sync_formats(self):
         old_extensions, new_extensions, paths = set(), set(), {}
         for row in range(self.formats.count()):
@@ -331,6 +346,14 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         ResizableDialog.__init__(self, window)
         self.bc_box.layout().setAlignment(self.cover, Qt.AlignCenter|Qt.AlignHCenter)
         self.cancel_all = False
+        base = unicode(self.author_sort.toolTip())
+        self.ok_aus_tooltip = '<p>' + textwrap.fill(base+'<br><br>'+
+                            _(' The green color indicates that the current '
+                    'author sort matches the current author'))
+        self.bad_aus_tooltip = '<p>'+textwrap.fill(base + '<br><br>'+
+                _(' The red color indicates that the current '
+                    'author sort does not match the current author'))
+
         if cancel_all:
             self.__abort_button = self.button_box.addButton(self.button_box.Abort)
             self.__abort_button.setToolTip(_('Abort the editing of all remaining books'))
@@ -375,6 +398,11 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                         self.remove_unused_series)
         QObject.connect(self.auto_author_sort, SIGNAL('clicked()'),
                         self.deduce_author_sort)
+        self.trim_cover_button.clicked.connect(self.trim_cover)
+        self.connect(self.author_sort, SIGNAL('textChanged(const QString&)'),
+                     self.author_sort_box_changed)
+        self.connect(self.authors, SIGNAL('editTextChanged(const QString&)'),
+                     self.authors_box_changed)
         self.connect(self.formats, SIGNAL('itemDoubleClicked(QListWidgetItem*)'),
                 self.show_format)
         self.connect(self.formats, SIGNAL('delete_format()'), self.remove_format)
@@ -406,9 +434,9 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         self.pubdate.setDate(QDate(pubdate.year, pubdate.month,
             pubdate.day))
         timestamp = db.timestamp(self.id, index_is_id=True)
-        self.orig_timestamp = timestamp
         self.date.setDate(QDate(timestamp.year, timestamp.month,
             timestamp.day))
+        self.orig_date = qt_to_dt(self.date.date())
 
         exts = self.db.formats(row)
         if exts:
@@ -467,6 +495,28 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
             for c in range(2, len(ans[i].widgets), 2):
                 w.setTabOrder(ans[i].widgets[c-1], ans[i].widgets[c+1])
 
+    def authors_box_changed(self, txt):
+        aus = unicode(txt)
+        aus = re.sub(r'\s+et al\.$', '', aus)
+        aus = self.db.author_sort_from_authors(string_to_authors(aus))
+        self.mark_author_sort(normal=(unicode(self.author_sort.text()) == aus))
+
+    def author_sort_box_changed(self, txt):
+        au = unicode(self.authors.text())
+        au = re.sub(r'\s+et al\.$', '', au)
+        au = self.db.author_sort_from_authors(string_to_authors(au))
+        self.mark_author_sort(normal=(au == txt))
+
+    def mark_author_sort(self, normal=True):
+        if normal:
+            col = 'rgb(0, 255, 0, 20%)'
+        else:
+            col = 'rgb(255, 0, 0, 20%)'
+        self.author_sort.setStyleSheet('QLineEdit { color: black; '
+                                       'background-color: %s; }'%col)
+        tt = self.ok_aus_tooltip if normal else self.bad_aus_tooltip
+        self.author_sort.setToolTip(tt)
+
     def validate_isbn(self, isbn):
         isbn = unicode(isbn).strip()
         if not isbn:
@@ -558,9 +608,16 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
 
     def edit_tags(self):
         if self.tags.text() != self.original_tags:
-            error_dialog(self, _('Cannot use tag editor'),
-                _('The tags editor cannot be used if you have modified the tags')).exec_()
-            return
+            if question_dialog(self, _('Tags changed'),
+                    _('You have changed the tags. In order to use the tags'
+                       ' editor, you must either discard or apply these '
+                       'changes'), show_copy_button=False,
+                    buttons=QMessageBox.Apply|QMessageBox.Discard,
+                    yes_button=QMessageBox.Apply):
+                self.apply_tags(commit=True, notify=True)
+                self.original_tags = unicode(self.tags.text())
+            else:
+                self.tags.setText(self.original_tags)
         d = TagEditor(self, self.db, self.row)
         d.exec_()
         if d.result() == QDialog.Accepted:
@@ -679,10 +736,13 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                                 self.series.setText(book.series)
                                 if book.series_index is not None:
                                     self.series_index.setValue(book.series_index)
+                        # Needed because of Qt focus bug on OS X
+                        self.fetch_cover_button.setFocus(Qt.OtherFocusReason)
         else:
             error_dialog(self, _('Cannot fetch metadata'),
                          _('You must specify at least one of ISBN, Title, '
                            'Authors or Publisher'), show=True)
+            self.title.setFocus(Qt.OtherFocusReason)
 
     def enable_series_index(self, *args):
         self.series_index.setEnabled(True)
@@ -711,6 +771,10 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                     self.series.setCurrentIndex(i)
                     break
 
+    def apply_tags(self, commit=False, notify=False):
+        self.db.set_tags(self.id, [x.strip() for x in
+            unicode(self.tags.text()).split(',')],
+                notify=notify, commit=commit)
 
     def accept(self):
         cf = getattr(self, 'cover_fetcher', None)
@@ -720,36 +784,38 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         try:
             if self.formats_changed:
                 self.sync_formats()
-            title = unicode(self.title.text())
+            title = unicode(self.title.text()).strip()
             self.db.set_title(self.id, title, notify=False)
-            au = unicode(self.authors.text())
+            au = unicode(self.authors.text()).strip()
             if au:
                 self.db.set_authors(self.id, string_to_authors(au), notify=False)
-            aus = unicode(self.author_sort.text())
+            aus = unicode(self.author_sort.text()).strip()
             if aus:
                 self.db.set_author_sort(self.id, aus, notify=False, commit=False)
             self.db.set_isbn(self.id,
-                             re.sub(r'[^0-9a-zA-Z]', '', unicode(self.isbn.text())),
+                             re.sub(r'[^0-9a-zA-Z]', '',
+                                 unicode(self.isbn.text()).strip()),
                              notify=False, commit=False)
             self.db.set_rating(self.id, 2*self.rating.value(), notify=False,
                                commit=False)
-            self.db.set_publisher(self.id, unicode(self.publisher.currentText()),
+            self.apply_tags()
+            self.db.set_publisher(self.id,
+                    unicode(self.publisher.currentText()).strip(),
                                   notify=False, commit=False)
-            self.db.set_tags(self.id, [x.strip() for x in
-                unicode(self.tags.text()).split(',')], notify=False, commit=False)
             self.db.set_series(self.id,
                     unicode(self.series.currentText()).strip(), notify=False,
                     commit=False)
             self.db.set_series_index(self.id, self.series_index.value(),
                                      notify=False, commit=False)
-            self.db.set_comment(self.id, unicode(self.comments.toPlainText()),
+            self.db.set_comment(self.id,
+                    unicode(self.comments.toPlainText()).strip(),
                                 notify=False, commit=False)
             d = self.pubdate.date()
             d = qt_to_dt(d)
             self.db.set_pubdate(self.id, d, notify=False, commit=False)
             d = self.date.date()
             d = qt_to_dt(d)
-            if d.date() != self.orig_timestamp.date():
+            if d != self.orig_date:
                 self.db.set_timestamp(self.id, d, notify=False, commit=False)
             self.db.commit()
 
@@ -766,7 +832,8 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
                 fname = err.filename if err.filename else 'file'
                 return error_dialog(self, _('Permission denied'),
                         _('Could not open %s. Is it being used by another'
-                        ' program?')%fname, show=True)
+                        ' program?')%fname, det_msg=traceback.format_exc(),
+                        show=True)
             raise
         self.save_state()
         QDialog.accept(self)
@@ -778,7 +845,7 @@ class MetadataSingleDialog(ResizableDialog, Ui_MetadataSingleDialog):
         if cf is not None and hasattr(cf, 'terminate'):
             cf.terminate()
             cf.wait()
-
+        self.save_state()
         QDialog.reject(self, *args)
 
     def read_state(self):
