@@ -6,15 +6,16 @@ __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
+import re
+
 from PyQt4.Qt import QComboBox, Qt, QLineEdit, QStringList, pyqtSlot, \
                      pyqtSignal, SIGNAL, QObject, QDialog, QCompleter, \
-                     QAction, QKeySequence
+                     QAction, QKeySequence, QTimer
 
 from calibre.gui2 import config
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.dialogs.saved_search_editor import SavedSearchEditor
 from calibre.gui2.dialogs.search import SearchDialog
-from calibre.utils.config import prefs
 from calibre.utils.search_query_parser import saved_searches
 
 class SearchLineEdit(QLineEdit):
@@ -72,6 +73,8 @@ class SearchBox2(QComboBox):
         self.normal_background = 'rgb(255, 255, 255, 0%)'
         self.line_edit = SearchLineEdit(self)
         self.setLineEdit(self.line_edit)
+        c = self.line_edit.completer()
+        c.setCompletionMode(c.PopupCompletion)
         self.line_edit.key_pressed.connect(self.key_pressed,
                 type=Qt.DirectConnection)
         self.line_edit.mouse_released.connect(self.mouse_released,
@@ -81,12 +84,15 @@ class SearchBox2(QComboBox):
         self.help_state = False
         self.as_you_type = True
         self.prev_search = ''
-        self.timer = None
+        self.timer = QTimer()
+        self.timer.setSingleShot(True)
+        self.timer.timeout.connect(self.timer_event, type=Qt.QueuedConnection)
         self.setInsertPolicy(self.NoInsert)
         self.setMaxCount(self.MAX_COUNT)
         self.setSizeAdjustPolicy(self.AdjustToMinimumContentsLengthWithIcon)
         self.setMinimumContentsLength(25)
         self._in_a_search = False
+        self.tool_tip_text = self.toolTip()
 
     def initialize(self, opt_name, colorize=False, help_text=_('Search')):
         self.as_you_type = config['search_as_you_type']
@@ -97,6 +103,7 @@ class SearchBox2(QComboBox):
         self.clear_to_help()
 
     def normalize_state(self):
+        self.setToolTip(self.tool_tip_text)
         if self.help_state:
             self.setEditText('')
             self.line_edit.setStyleSheet(
@@ -109,15 +116,13 @@ class SearchBox2(QComboBox):
                     self.normal_background)
 
     def clear_to_help(self):
+        self.setToolTip(self.tool_tip_text)
         if self.help_state:
             return
         self.help_state = True
         self.search.emit('')
         self._in_a_search = False
         self.setEditText(self.help_text)
-        if self.timer is not None: # Turn off any timers that got started in setEditText
-            self.killTimer(self.timer)
-            self.timer = None
         self.line_edit.home(False)
         self.line_edit.setStyleSheet(
                 'QLineEdit { color: gray; background-color: %s; }' %
@@ -131,6 +136,9 @@ class SearchBox2(QComboBox):
         self.clear_to_help()
 
     def search_done(self, ok):
+        if isinstance(ok, basestring):
+            self.setToolTip(ok)
+            ok = False
         if not unicode(self.currentText()).strip():
             return self.clear_to_help()
         self._in_a_search = ok
@@ -140,24 +148,29 @@ class SearchBox2(QComboBox):
         self.line_edit.setStyleSheet('QLineEdit { color: black; background-color: %s; }' % col)
 
     def key_pressed(self, event):
+        k = event.key()
+        if k in (Qt.Key_Left, Qt.Key_Right, Qt.Key_Up, Qt.Key_Down,
+                Qt.Key_Home, Qt.Key_End, Qt.Key_PageUp, Qt.Key_PageDown):
+            return
         self.normalize_state()
         if self._in_a_search:
             self.emit(SIGNAL('changed()'))
             self._in_a_search = False
         if event.key() in (Qt.Key_Return, Qt.Key_Enter):
             self.do_search()
-        self.timer = self.startTimer(self.__class__.INTERVAL)
+        if self.as_you_type:
+            self.timer.start(1500)
 
     def mouse_released(self, event):
         self.normalize_state()
-        if self.as_you_type:
-            self.timer = self.startTimer(self.__class__.INTERVAL)
+        # Dont trigger a search since it make
+        # re-positioning the cursor using the mouse
+        # impossible
+        #if self.as_you_type:
+        #    self.timer.start(1500)
 
-    def timerEvent(self, event):
-        self.killTimer(event.timerId())
-        if event.timerId() == self.timer:
-            self.timer = None
-            self.do_search()
+    def timer_event(self):
+        self.do_search()
 
     def history_selected(self, text):
         self.emit(SIGNAL('changed()'))
@@ -170,7 +183,7 @@ class SearchBox2(QComboBox):
             return ''
         return text
 
-    def do_search(self):
+    def do_search(self, *args):
         text = unicode(self.currentText()).strip()
         if not text or text == self.help_text:
             return self.clear()
@@ -211,9 +224,6 @@ class SearchBox2(QComboBox):
             return
         self.normalize_state()
         self.setEditText(txt)
-        if self.timer is not None: # Turn off any timers that got started in setEditText
-            self.killTimer(self.timer)
-            self.timer = None
         self.search.emit(txt)
         self.line_edit.end(False)
         self.initial_state = False
@@ -257,8 +267,7 @@ class SavedSearchBox(QComboBox):
         self.setMinimumContentsLength(10)
         self.tool_tip_text = self.toolTip()
 
-    def initialize(self, _saved_searches, _search_box, colorize=False, help_text=_('Search')):
-        self.saved_searches = _saved_searches
+    def initialize(self, _search_box, colorize=False, help_text=_('Search')):
         self.search_box = _search_box
         self.help_text = help_text
         self.colorize = colorize
@@ -300,11 +309,11 @@ class SavedSearchBox(QComboBox):
         self.normalize_state()
         self.search_box.set_search_string(u'search:"%s"' % qname)
         self.setEditText(qname)
-        self.setToolTip(self.saved_searches.lookup(qname))
+        self.setToolTip(saved_searches().lookup(qname))
 
     def initialize_saved_search_names(self):
         self.clear()
-        qnames = self.saved_searches.names()
+        qnames = saved_searches().names()
         self.addItems(qnames)
         self.setCurrentIndex(-1)
 
@@ -317,10 +326,10 @@ class SavedSearchBox(QComboBox):
         idx = self.currentIndex
         if idx < 0:
             return
-        ss = self.saved_searches.lookup(unicode(self.currentText()))
+        ss = saved_searches().lookup(unicode(self.currentText()))
         if ss is None:
             return
-        self.saved_searches.delete(unicode(self.currentText()))
+        saved_searches().delete(unicode(self.currentText()))
         self.clear_to_help()
         self.search_box.clear_to_help()
         self.emit(SIGNAL('changed()'))
@@ -330,8 +339,8 @@ class SavedSearchBox(QComboBox):
         name = unicode(self.currentText())
         if self.help_state or not name.strip():
             name = unicode(self.search_box.text()).replace('"', '')
-        self.saved_searches.delete(name)
-        self.saved_searches.add(name, unicode(self.search_box.text()))
+        saved_searches().delete(name)
+        saved_searches().add(name, unicode(self.search_box.text()))
         # now go through an initialization cycle to ensure that the combobox has
         # the new search in it, that it is selected, and that the search box
         # references the new search instead of the text in the search.
@@ -346,7 +355,7 @@ class SavedSearchBox(QComboBox):
         idx = self.currentIndex();
         if idx < 0:
             return
-        self.search_box.set_search_string(self.saved_searches.lookup(unicode(self.currentText())))
+        self.search_box.set_search_string(saved_searches().lookup(unicode(self.currentText())))
 
 class SearchBoxMixin(object):
 
@@ -363,11 +372,15 @@ class SearchBoxMixin(object):
         self.search.setMaximumWidth(self.width()-150)
         self.action_focus_search = QAction(self)
         shortcuts = QKeySequence.keyBindings(QKeySequence.Find)
-        shortcuts = list(shortcuts) + [QKeySequence('/')]
+        shortcuts = list(shortcuts) + [QKeySequence('/'), QKeySequence('Alt+S')]
         self.action_focus_search.setShortcuts(shortcuts)
         self.action_focus_search.triggered.connect(lambda x:
                 self.search.setFocus(Qt.OtherFocusReason))
         self.addAction(self.action_focus_search)
+        self.search.setStatusTip(re.sub(r'<\w+>', ' ',
+            unicode(self.search.toolTip())))
+        self.advanced_search_button.setStatusTip(self.advanced_search_button.toolTip())
+        self.clear_button.setStatusTip(self.clear_button.toolTip())
 
     def search_box_cleared(self):
         self.tags_view.clear()
@@ -375,10 +388,11 @@ class SearchBoxMixin(object):
         self.set_number_of_books_shown()
 
     def search_box_changed(self):
+        self.saved_search.clear_to_help()
         self.tags_view.clear()
 
     def do_advanced_search(self, *args):
-        d = SearchDialog(self)
+        d = SearchDialog(self, self.library_view.model().db)
         if d.exec_() == QDialog.Accepted:
             self.search.set_search_string(d.search_string())
 
@@ -388,7 +402,7 @@ class SavedSearchBoxMixin(object):
         self.connect(self.saved_search, SIGNAL('changed()'), self.saved_searches_changed)
         self.saved_searches_changed()
         self.connect(self.clear_button, SIGNAL('clicked()'), self.saved_search.clear_to_help)
-        self.saved_search.initialize(saved_searches, self.search, colorize=True,
+        self.saved_search.initialize(self.search, colorize=True,
                 help_text=_('Saved Searches'))
         self.connect(self.save_search_button, SIGNAL('clicked()'),
                 self.saved_search.save_search_button_clicked)
@@ -396,24 +410,23 @@ class SavedSearchBoxMixin(object):
                 self.saved_search.delete_search_button_clicked)
         self.connect(self.copy_search_button, SIGNAL('clicked()'),
                 self.saved_search.copy_search_button_clicked)
-
+        self.saved_search.setToolTip(
+            _('Choose saved search or enter name for new saved search'))
+        self.saved_search.setStatusTip(self.saved_search.toolTip())
+        for x in ('copy', 'save', 'delete'):
+            b = getattr(self, x+'_search_button')
+            b.setStatusTip(b.toolTip())
 
     def saved_searches_changed(self):
-        p = prefs['saved_searches'].keys()
-        p.sort()
+        p = sorted(saved_searches().names(), cmp=lambda x,y: cmp(x.lower(), y.lower()))
         t = unicode(self.search_restriction.currentText())
         self.search_restriction.clear() # rebuild the restrictions combobox using current saved searches
         self.search_restriction.addItem('')
         self.tags_view.recount()
         for s in p:
             self.search_restriction.addItem(s)
-        if t:
-            if t in p: # redo the current restriction, if there was one
-                self.search_restriction.setCurrentIndex(self.search_restriction.findText(t))
-                # self.tags_view.set_search_restriction(t)
-            else:
-                self.search_restriction.setCurrentIndex(0)
-                self.apply_search_restriction('')
+        if t: # redo the search restriction if there was one
+            self.apply_named_search_restriction(t)
 
     def do_saved_search_edit(self, search):
         d = SavedSearchEditor(self, search)
