@@ -17,9 +17,10 @@ from PyQt4.Qt import Qt, QTreeView, QApplication, pyqtSignal, \
 
 from calibre.ebooks.metadata import title_sort
 from calibre.gui2 import config, NONE
-from calibre.library.field_metadata import TagsIcons
+from calibre.library.field_metadata import TagsIcons, category_icon_map
 from calibre.utils.search_query_parser import saved_searches
 from calibre.gui2 import error_dialog
+from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.dialogs.tag_categories import TagCategories
 from calibre.gui2.dialogs.tag_list_editor import TagListEditor
 from calibre.gui2.dialogs.edit_authors_dialog import EditAuthorsDialog
@@ -66,6 +67,7 @@ class TagsView(QTreeView): # {{{
     author_sort_edit    = pyqtSignal(object, object)
     tag_item_renamed    = pyqtSignal()
     search_item_renamed = pyqtSignal()
+    drag_drop_finished  = pyqtSignal(object)
 
     def __init__(self, parent=None):
         QTreeView.__init__(self, parent=None)
@@ -79,12 +81,17 @@ class TagsView(QTreeView): # {{{
         self.setHeaderHidden(True)
         self.setItemDelegate(TagDelegate(self))
         self.made_connections = False
+        self.setAcceptDrops(True)
+        self.setDragDropMode(self.DropOnly)
+        self.setDropIndicatorShown(True)
+        self.setAutoExpandDelay(500)
 
     def set_database(self, db, tag_match, sort_by):
         self.hidden_categories = config['tag_browser_hidden_categories']
         self._model = TagsModel(db, parent=self,
                                 hidden_categories=self.hidden_categories,
-                                search_restriction=None)
+                                search_restriction=None,
+                                drag_drop_finished=self.drag_drop_finished)
         self.sort_by = sort_by
         self.tag_match = tag_match
         self.db = db
@@ -272,7 +279,8 @@ class TagsView(QTreeView): # {{{
         try:
             self._model = TagsModel(self.db, parent=self,
                                     hidden_categories=self.hidden_categories,
-                                    search_restriction=self.search_restriction)
+                                    search_restriction=self.search_restriction,
+                                    drag_drop_finished=self.drag_drop_finished)
             self.setModel(self._model)
         except:
             # The DB must be gone. Set the model to None and hope that someone
@@ -326,6 +334,8 @@ class TagTreeItem(object): # {{{
         self.children.append(child)
 
     def data(self, role):
+        if role == Qt.UserRole:
+            return self
         if self.type == self.TAG:
             return self.tag_data(role)
         if self.type == self.CATEGORY:
@@ -365,24 +375,20 @@ class TagTreeItem(object): # {{{
 
 class TagsModel(QAbstractItemModel): # {{{
 
-    def __init__(self, db, parent, hidden_categories=None, search_restriction=None):
+    def __init__(self, db, parent, hidden_categories=None,
+            search_restriction=None, drag_drop_finished=None):
         QAbstractItemModel.__init__(self, parent)
 
         # must do this here because 'QPixmap: Must construct a QApplication
         # before a QPaintDevice'. The ':' in front avoids polluting either the
         # user-defined categories (':' at end) or columns namespaces (no ':').
-        self.category_icon_map = TagsIcons({
-                    'authors'   : QIcon(I('user_profile.png')),
-                    'series'    : QIcon(I('series.png')),
-                    'formats'   : QIcon(I('book.png')),
-                    'publisher' : QIcon(I('publisher.png')),
-                    'rating'    : QIcon(I('star.png')),
-                    'news'      : QIcon(I('news.png')),
-                    'tags'      : QIcon(I('tags.png')),
-                    ':custom'   : QIcon(I('column.png')),
-                    ':user'     : QIcon(I('drawer.png')),
-                    'search'    : QIcon(I('search.png'))})
+        iconmap = {}
+        for key in category_icon_map:
+            iconmap[key] = QIcon(I(category_icon_map[key]))
+        self.category_icon_map = TagsIcons(iconmap)
+
         self.categories_with_ratings = ['authors', 'series', 'publisher', 'tags']
+        self.drag_drop_finished = drag_drop_finished
 
         self.icon_state_map = [None, QIcon(I('plus.png')), QIcon(I('minus.png'))]
         self.db = db
@@ -415,6 +421,79 @@ class TagsModel(QAbstractItemModel): # {{{
                     tag.avg_rating = None
                 TagTreeItem(parent=c, data=tag, icon_map=self.icon_state_map)
 
+    def mimeTypes(self):
+        return ["application/calibre+from_library"]
+
+    def dropMimeData(self, md, action, row, column, parent):
+        if not md.hasFormat("application/calibre+from_library") or \
+                action != Qt.CopyAction:
+            return False
+        idx = parent
+        if idx.isValid():
+            node = self.data(idx, Qt.UserRole)
+            if node.type == TagTreeItem.TAG:
+                fm = self.db.metadata_for_field(node.tag.category)
+                if node.tag.category in \
+                    ('tags', 'series', 'authors', 'rating', 'publisher') or \
+                    (fm['is_custom'] and \
+                        fm['datatype'] in ['text', 'rating', 'series']):
+                    mime = 'application/calibre+from_library'
+                    ids = list(map(int, str(md.data(mime)).split()))
+                    self.handle_drop(node, ids)
+                    return True
+        return False
+
+
+    def handle_drop(self, on_node, ids):
+        #print 'Dropped ids:', ids, on_node.tag
+        key = on_node.tag.category
+        if (key == 'authors' and len(ids) >= 5):
+            if not confirm('<p>'+_('Changing the authors for several books can '
+                           'take a while. Are you sure?')
+                        +'</p>', 'tag_browser_drop_authors', self.parent()):
+                return
+        elif len(ids) > 15:
+            if not confirm('<p>'+_('Changing the metadata for that many books '
+                           'can take a while. Are you sure?')
+                        +'</p>', 'tag_browser_many_changes', self.parent()):
+                return
+
+        fm = self.db.metadata_for_field(key)
+        is_multiple = fm['is_multiple']
+        val = on_node.tag.name
+        for id in ids:
+            mi = self.db.get_metadata(id, index_is_id=True)
+
+            # Prepare to ignore the author, unless it is changed. Title is
+            # always ignored -- see the call to set_metadata
+            set_authors = False
+
+            # Author_sort cannot change explicitly. Changing the author might
+            # change it.
+            mi.author_sort = None # Never will change by itself.
+
+            if key == 'authors':
+                mi.authors = [val]
+                set_authors=True
+            elif fm['datatype'] == 'rating':
+                mi.set(key, len(val) * 2)
+            elif fm['is_custom'] and fm['datatype'] == 'series':
+                mi.set(key, val, extra=1.0)
+            elif is_multiple:
+                new_val = mi.get(key, [])
+                if val in new_val:
+                    # Fortunately, only one field can change, so the continue
+                    # won't break anything
+                    continue
+                new_val.append(val)
+                mi.set(key, new_val)
+            else:
+                mi.set(key, val)
+            self.db.set_metadata(id, mi, set_title=False,
+                                 set_authors=set_authors, commit=False)
+        self.db.commit()
+        self.drag_drop_finished.emit(ids)
+
     def set_search_restriction(self, s):
         self.search_restriction = s
 
@@ -505,14 +584,20 @@ class TagsModel(QAbstractItemModel): # {{{
         key = item.parent.category_key
         # make certain we know about the item's category
         if key not in self.db.field_metadata:
-            return
+            return False
+        if key == 'authors':
+            if val.find('&') >= 0:
+                error_dialog(self.tags_view, _('Invalid author name'),
+                        _('Author names cannot contain & characters.')).exec_()
+                return False
         if key == 'search':
             if val in saved_searches().names():
                 error_dialog(self.tags_view, _('Duplicate search name'),
                     _('The saved search name %s is already used.')%val).exec_()
                 return False
             saved_searches().rename(unicode(item.data(role).toString()), val)
-            self.tags_view.search_item_renamed.emit()
+            item.tag.name = val
+            self.tags_view.search_item_renamed.emit() # Does a refresh
         else:
             if key == 'series':
                 self.db.rename_series(item.tag.id, val)
@@ -526,8 +611,8 @@ class TagsModel(QAbstractItemModel): # {{{
                 self.db.rename_custom_item(item.tag.id, val,
                                     label=self.db.field_metadata[key]['label'])
             self.tags_view.tag_item_renamed.emit()
-        item.tag.name = val
-        self.refresh() # Should work, because no categories can have disappeared
+            item.tag.name = val
+            self.refresh() # Should work, because no categories can have disappeared
         if path:
             idx = self.index_for_path(path)
             if idx.isValid():
@@ -538,8 +623,21 @@ class TagsModel(QAbstractItemModel): # {{{
     def headerData(self, *args):
         return NONE
 
-    def flags(self, *args):
-        return Qt.ItemIsEnabled|Qt.ItemIsSelectable|Qt.ItemIsEditable
+    def flags(self, index, *args):
+        ans = Qt.ItemIsEnabled|Qt.ItemIsSelectable|Qt.ItemIsEditable
+        if index.isValid():
+            node = self.data(index, Qt.UserRole)
+            if node.type == TagTreeItem.TAG:
+                fm = self.db.metadata_for_field(node.tag.category)
+                if node.tag.category in \
+                    ('tags', 'series', 'authors', 'rating', 'publisher') or \
+                    (fm['is_custom'] and \
+                        fm['datatype'] in ['text', 'rating', 'series']):
+                    ans |= Qt.ItemIsDropEnabled
+        return ans
+
+    def supportedDropActions(self):
+        return Qt.CopyAction
 
     def path_for_index(self, index):
         ans = []
@@ -669,7 +767,8 @@ class TagBrowserMixin(object): # {{{
         self.tags_view.saved_search_edit.connect(self.do_saved_search_edit)
         self.tags_view.author_sort_edit.connect(self.do_author_sort_edit)
         self.tags_view.tag_item_renamed.connect(self.do_tag_item_renamed)
-        self.tags_view.search_item_renamed.connect(self.saved_search.clear_to_help)
+        self.tags_view.search_item_renamed.connect(self.saved_searches_changed)
+        self.tags_view.drag_drop_finished.connect(self.drag_drop_finished)
         self.edit_categories.clicked.connect(lambda x:
                 self.do_user_categories_edit())
 
@@ -719,11 +818,11 @@ class TagBrowserMixin(object): # {{{
                 rename_func = partial(db.rename_custom_item, label=cc_label)
                 delete_func = partial(db.delete_custom_item_using_id, label=cc_label)
             if rename_func:
+                for item in to_delete:
+                    delete_func(item)
                 for text in to_rename:
                         for old_id in to_rename[text]:
                             rename_func(old_id, new_name=unicode(text))
-                for item in to_delete:
-                    delete_func(item)
 
             # Clean up everything, as information could have changed for many books.
             self.library_view.model().refresh()
@@ -751,6 +850,9 @@ class TagBrowserMixin(object): # {{{
             self.library_view.model().refresh()
             self.tags_view.recount()
 
+    def drag_drop_finished(self, ids):
+        self.library_view.model().refresh_ids(ids)
+
 # }}}
 
 class TagBrowserWidget(QWidget): # {{{
@@ -759,6 +861,7 @@ class TagBrowserWidget(QWidget): # {{{
         QWidget.__init__(self, parent)
         self._layout = QVBoxLayout()
         self.setLayout(self._layout)
+        self._layout.setContentsMargins(0,0,0,0)
 
         parent.tags_view = TagsView(parent)
         self._layout.addWidget(parent.tags_view)
