@@ -19,14 +19,15 @@ from PyQt4.Qt import Qt, SIGNAL, QTimer, \
                      QMessageBox, QHelpEvent
 
 from calibre import  prints
-from calibre.constants import __appname__, isosx
+from calibre.constants import __appname__, isosx, DEBUG
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.config import prefs, dynamic
 from calibre.utils.ipc.server import Server
 from calibre.library.database2 import LibraryDatabase2
 from calibre.customize.ui import interface_actions
 from calibre.gui2 import error_dialog, GetMetadata, open_local_file, \
-        gprefs, max_available_height, config, info_dialog, Dispatcher
+        gprefs, max_available_height, config, info_dialog, Dispatcher, \
+        question_dialog
 from calibre.gui2.cover_flow import CoverFlowMixin
 from calibre.gui2.widgets import ProgressIndicator
 from calibre.gui2.update import UpdateMixin
@@ -38,6 +39,7 @@ from calibre.gui2.init import LibraryViewMixin, LayoutMixin
 from calibre.gui2.search_box import SearchBoxMixin, SavedSearchBoxMixin
 from calibre.gui2.search_restriction_mixin import SearchRestrictionMixin
 from calibre.gui2.tag_view import TagBrowserMixin
+from calibre.utils.ordered_dict import OrderedDict
 
 
 class Listener(Thread): # {{{
@@ -97,7 +99,7 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
         MainWindow.__init__(self, opts, parent)
         self.opts = opts
         self.device_connected = None
-        acmap = {}
+        acmap = OrderedDict()
         for action in interface_actions():
             mod, cls = action.actual_plugin.split(':')
             ac = getattr(__import__(mod, fromlist=['1'], level=0), cls)(self,
@@ -180,9 +182,8 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
         self.connect(self.donate_action, SIGNAL('triggered(bool)'), self.donate)
         self.connect(self.restore_action, SIGNAL('triggered()'),
                         self.show_windows)
-        self.connect(self.system_tray_icon,
-                     SIGNAL('activated(QSystemTrayIcon::ActivationReason)'),
-                     self.system_tray_icon_activated)
+        self.system_tray_icon.activated.connect(
+            self.system_tray_icon_activated)
 
 
         ####################### Start spare job server ########################
@@ -249,7 +250,6 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
 
         self.keyboard_interrupt.connect(self.quit, type=Qt.QueuedConnection)
 
-
         self.read_settings()
         self.finalize_layout()
         if self.tool_bar.showing_donate:
@@ -300,6 +300,16 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
             else:
                 self.show_windows()
 
+    @property
+    def is_minimized_to_tray(self):
+        return getattr(self, '__systray_minimized', False)
+
+    def ask_a_yes_no_question(self, title, msg, **kwargs):
+        awu = kwargs.pop('ans_when_user_unavailable', True)
+        if self.is_minimized_to_tray:
+            return awu
+        return question_dialog(self, title, msg, **kwargs)
+
     def hide_windows(self):
         for window in QApplication.topLevelWidgets():
             if isinstance(window, (MainWindow, QDialog)) and \
@@ -359,7 +369,13 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
 
     def library_moved(self, newloc):
         if newloc is None: return
+        try:
+            olddb = self.library_view.model().db
+        except:
+            olddb = None
         db = LibraryDatabase2(newloc)
+        if self.content_server is not None:
+            self.content_server.set_database(db)
         self.library_path = newloc
         self.book_on_device(None, reset=True)
         db.set_book_on_device_func(self.book_on_device)
@@ -379,6 +395,19 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
         self.apply_named_search_restriction('') # reset restriction to null
         self.saved_searches_changed() # reload the search restrictions combo box
         self.apply_named_search_restriction(db.prefs['gui_restriction'])
+        if olddb is not None:
+            try:
+                olddb.conn.close()
+            except:
+                import traceback
+                traceback.print_exc()
+        if self.device_connected:
+            self.set_books_in_library(self.booklists(), reset=True)
+            self.refresh_ondevice()
+            self.memory_view.reset()
+            self.card_a_view.reset()
+            self.card_b_view.reset()
+
 
     def set_window_title(self):
         self.setWindowTitle(__appname__ + u' - ||%s||'%self.iactions['Choose Library'].library_name())
@@ -408,31 +437,34 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
     def job_exception(self, job):
         if not hasattr(self, '_modeless_dialogs'):
             self._modeless_dialogs = []
+        minz = self.is_minimized_to_tray
         if self.isVisible():
             for x in list(self._modeless_dialogs):
                 if not x.isVisible():
                     self._modeless_dialogs.remove(x)
         try:
             if 'calibre.ebooks.DRMError' in job.details:
-                d = error_dialog(self, _('Conversion Error'),
-                    _('<p>Could not convert: %s<p>It is a '
-                      '<a href="%s">DRM</a>ed book. You must first remove the '
-                      'DRM using third party tools.')%\
-                        (job.description.split(':')[-1],
-                            'http://bugs.calibre-ebook.com/wiki/DRM'))
-                d.setModal(False)
-                d.show()
-                self._modeless_dialogs.append(d)
+                if not minz:
+                    d = error_dialog(self, _('Conversion Error'),
+                        _('<p>Could not convert: %s<p>It is a '
+                        '<a href="%s">DRM</a>ed book. You must first remove the '
+                        'DRM using third party tools.')%\
+                            (job.description.split(':')[-1],
+                                'http://bugs.calibre-ebook.com/wiki/DRM'))
+                    d.setModal(False)
+                    d.show()
+                    self._modeless_dialogs.append(d)
                 return
             if 'calibre.web.feeds.input.RecipeDisabled' in job.details:
-                msg = job.details
-                msg = msg[msg.find('calibre.web.feeds.input.RecipeDisabled:'):]
-                msg = msg.partition(':')[-1]
-                d = error_dialog(self, _('Recipe Disabled'),
-                    '<p>%s</p>'%msg)
-                d.setModal(False)
-                d.show()
-                self._modeless_dialogs.append(d)
+                if not minz:
+                    msg = job.details
+                    msg = msg[msg.find('calibre.web.feeds.input.RecipeDisabled:'):]
+                    msg = msg.partition(':')[-1]
+                    d = error_dialog(self, _('Recipe Disabled'),
+                        '<p>%s</p>'%msg)
+                    d.setModal(False)
+                    d.show()
+                    self._modeless_dialogs.append(d)
                 return
         except:
             pass
@@ -442,12 +474,13 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
             prints(job.details, file=sys.stderr)
         except:
             pass
-        d = error_dialog(self, _('Conversion Error'),
-                _('<b>Failed</b>')+': '+unicode(job.description),
-                det_msg=job.details)
-        d.setModal(False)
-        d.show()
-        self._modeless_dialogs.append(d)
+        if not minz:
+            d = error_dialog(self, _('Conversion Error'),
+                    _('<b>Failed</b>')+': '+unicode(job.description),
+                    det_msg=job.details)
+            d.setModal(False)
+            d.show()
+            self._modeless_dialogs.append(d)
 
     def read_settings(self):
         geometry = config['main_window_geometry']
@@ -532,6 +565,10 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
             # Save the current field_metadata for applications like calibre2opds
             # Goes here, because if cf is valid, db is valid.
             db.prefs['field_metadata'] = db.field_metadata.all_metadata()
+            db.commit_dirty_cache()
+            if DEBUG and db.gm_count > 0:
+                print 'get_metadata cache: {0:d} calls, {1:4.2f}% misses'.format(
+                        db.gm_count, (db.gm_missed*100.0)/db.gm_count)
         for action in self.iactions.values():
             if not action.shutting_down():
                 return
@@ -547,6 +584,10 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
         cc = self.library_view.model().cover_cache
         if cc is not None:
             cc.stop()
+        mb = self.library_view.model().metadata_backup
+        if mb is not None:
+            mb.stop()
+
         self.hide_windows()
         self.emailer.stop()
         try:
@@ -557,9 +598,11 @@ class Main(MainWindow, MainWindowMixin, DeviceMixin, # {{{
                     s.exit()
             except:
                 pass
-            time.sleep(2)
         except KeyboardInterrupt:
             pass
+        time.sleep(2)
+        if mb is not None:
+            mb.flush()
         self.hide_windows()
         return True
 
