@@ -80,7 +80,7 @@ class ConfigDialog(QDialog, Ui_Dialog):
         self.css.setPlainText(opts.user_css)
         self.css.setToolTip(_('Set the user CSS stylesheet. This can be used to customize the look of all books.'))
         self.max_view_width.setValue(opts.max_view_width)
-        pats = [os.path.basename(x).split('.')[0] for x in
+        pats = [os.path.basename(x).split('.')[0].replace('-', '_') for x in
             glob.glob(P('viewer/hyphenate/patterns/*.js',
                 allow_user_override=False))]
         names = list(map(get_language, pats))
@@ -92,7 +92,7 @@ class ConfigDialog(QDialog, Ui_Dialog):
         try:
             idx = pats.index(opts.hyphenate_default_lang)
         except ValueError:
-            idx = pats.index('en')
+            idx = pats.index('en_us')
         idx = self.hyphenate_default_lang.findText(names[idx])
         self.hyphenate_default_lang.setCurrentIndex(idx)
         self.hyphenate.setChecked(opts.hyphenate)
@@ -143,13 +143,14 @@ class Document(QWebPage):
             self.set_font_settings()
             self.set_user_stylesheet()
             self.misc_config()
-            self.triggerAction(QWebPage.Reload)
+            self.after_load()
 
     def __init__(self, shortcuts, parent=None):
         QWebPage.__init__(self, parent)
         self.setObjectName("py_bridge")
         self.debug_javascript = False
         self.current_language = None
+        self.loaded_javascript = False
 
         self.setLinkDelegationPolicy(self.DelegateAllLinks)
         self.scroll_marks = []
@@ -175,9 +176,9 @@ class Document(QWebPage):
         self.set_user_stylesheet()
         self.misc_config()
 
-        # Load jQuery
-        self.connect(self.mainFrame(), SIGNAL('javaScriptWindowObjectCleared()'),
-                     self.load_javascript_libraries)
+        # Load javascript
+        self.mainFrame().javaScriptWindowObjectCleared.connect(
+                self.add_window_objects)
 
     def set_user_stylesheet(self):
         raw = config().parse().user_css
@@ -196,16 +197,20 @@ class Document(QWebPage):
         if self.do_fit_images:
             self.javascript('setup_image_scaling_handlers()')
 
+    def add_window_objects(self):
+        self.mainFrame().addToJavaScriptWindowObject("py_bridge", self)
+        self.loaded_javascript = False
+
     def load_javascript_libraries(self):
         global bookmarks, referencing, hyphenation, jquery, jquery_scrollTo, hyphenator, images
-        self.mainFrame().addToJavaScriptWindowObject("py_bridge", self)
+        if self.loaded_javascript:
+            return
+        self.loaded_javascript = True
         if jquery is None:
             jquery = P('content_server/jquery.js', data=True)
+        self.javascript(jquery)
         if jquery_scrollTo is None:
             jquery_scrollTo = P('viewer/jquery_scrollTo.js', data=True)
-        if hyphenator is None:
-            hyphenator = P('viewer/hyphenate/Hyphenator.js', data=True).decode('utf-8')
-        self.javascript(jquery)
         self.javascript(jquery_scrollTo)
         if bookmarks is None:
             bookmarks = P('viewer/bookmarks.js', data=True)
@@ -223,12 +228,17 @@ class Document(QWebPage):
         lang = self.current_language
         if not lang:
             lang = default_lang
-        lang = lang.lower()[:2]
+        def lang_name(l):
+            if l == 'en':
+                l = 'en-us'
+            return l.lower().replace('_', '-')
+        if hyphenator is None:
+            hyphenator = P('viewer/hyphenate/Hyphenator.js', data=True).decode('utf-8')
         self.javascript(hyphenator)
-        p = P('viewer/hyphenate/patterns/%s.js'%lang)
+        p = P('viewer/hyphenate/patterns/%s.js'%lang_name(lang))
         if not os.path.exists(p):
             lang = default_lang
-            p = P('viewer/hyphenate/patterns/%s.js'%lang)
+            p = P('viewer/hyphenate/patterns/%s.js'%lang_name(lang))
         self.javascript(open(p, 'rb').read().decode('utf-8'))
         self.loaded_lang = lang
 
@@ -242,6 +252,11 @@ class Document(QWebPage):
         if self.hyphenate:
             self.javascript('do_hyphenation("%s")'%self.loaded_lang)
 
+    def after_load(self):
+        self.set_bottom_padding(0)
+        self.fit_images()
+        self.init_hyphenate()
+
     @pyqtSignature("QString")
     def debug(self, msg):
         prints(msg)
@@ -256,6 +271,9 @@ class Document(QWebPage):
         self.javascript('goto_reference("%s")'%ref)
 
     def goto_bookmark(self, bm):
+        bm = bm.strip()
+        if bm.startswith('>'):
+            bm = bm[1:].strip()
         self.javascript('scroll_to_bookmark("%s")'%bm)
 
     def javascript(self, string, typ=None):
@@ -641,8 +659,8 @@ class DocumentView(QWebView):
             # An <iframe> finished loading
             return
         self.loading_url = None
-        self.document.set_bottom_padding(0)
-        self.document.fit_images()
+        self.document.load_javascript_libraries()
+        self.document.after_load()
         self._size_hint = self.document.mainFrame().contentsSize()
         scrolled = False
         if self.to_bottom:
@@ -804,6 +822,7 @@ class DocumentView(QWebView):
     def wheelEvent(self, event):
         if event.delta() < -14:
             if self.document.at_bottom:
+                self.scroll_by(y=15) # at_bottom can lie on windows
                 if self.manager is not None:
                     self.manager.next_document()
                     event.accept()
