@@ -6,7 +6,7 @@ __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import re, itertools, functools
+import re, itertools, time, traceback
 from itertools import repeat
 from datetime import timedelta
 from threading import Thread, RLock
@@ -19,14 +19,112 @@ from calibre.utils.date import parse_date, now, UNDEFINED_DATE
 from calibre.utils.search_query_parser import SearchQueryParser
 from calibre.utils.pyparsing import ParseException
 from calibre.ebooks.metadata import title_sort
-from calibre import fit_image
+from calibre.ebooks.metadata.opf2 import metadata_to_opf
+from calibre import fit_image, prints
 
-class CoverCache(Thread):
+class MetadataBackup(Thread): # {{{
+    '''
+    Continuously backup changed metadata into OPF files
+    in the book directory. This class runs in its own
+    thread and makes sure that the actual file write happens in the
+    GUI thread to prevent Windows' file locking from causing problems.
+    '''
 
     def __init__(self, db):
         Thread.__init__(self)
         self.daemon = True
         self.db = db
+        self.keep_running = True
+        from calibre.gui2 import FunctionDispatcher
+        self.do_write = FunctionDispatcher(self.write)
+        self.get_metadata_for_dump = FunctionDispatcher(db.get_metadata_for_dump)
+        self.clear_dirtied = FunctionDispatcher(db.clear_dirtied)
+        self.set_dirtied = FunctionDispatcher(db.dirtied)
+        self.in_limbo = None
+
+    def stop(self):
+        self.keep_running = False
+
+    def run(self):
+        while self.keep_running:
+            self.in_limbo = None
+            try:
+                time.sleep(0.5) # Limit to two per second
+                id_ = self.db.dirtied_queue.get(True, 1.45)
+            except Empty:
+                continue
+            except:
+                # Happens during interpreter shutdown
+                break
+
+            try:
+                path, mi = self.get_metadata_for_dump(id_)
+            except:
+                prints('Failed to get backup metadata for id:', id_, 'once')
+                traceback.print_exc()
+                time.sleep(2)
+                try:
+                    path, mi = self.get_metadata_for_dump(id_)
+                except:
+                    prints('Failed to get backup metadata for id:', id_, 'again, giving up')
+                    traceback.print_exc()
+                    continue
+
+            # at this point the dirty indication is off
+
+            if mi is None:
+                continue
+            self.in_limbo = id_
+
+            # Give the GUI thread a chance to do something. Python threads don't
+            # have priorities, so this thread would naturally keep the processor
+            # until some scheduling event happens. The sleep makes such an event
+            time.sleep(0.1)
+            try:
+                raw = metadata_to_opf(mi)
+            except:
+                self.set_dirtied([id_])
+                prints('Failed to convert to opf for id:', id_)
+                traceback.print_exc()
+                continue
+
+            time.sleep(0.1) # Give the GUI thread a chance to do something
+            try:
+                self.do_write(path, raw)
+            except:
+                prints('Failed to write backup metadata for id:', id_, 'once')
+                time.sleep(2)
+                try:
+                    self.do_write(path, raw)
+                except:
+                    self.set_dirtied([id_])
+                    prints('Failed to write backup metadata for id:', id_,
+                            'again, giving up')
+                    continue
+        self.in_limbo = None
+
+    def flush(self):
+        'Used during shutdown to ensure that a dirtied book is not missed'
+        if self.in_limbo is not None:
+            try:
+                self.db.dirtied([self.in_limbo])
+            except:
+                traceback.print_exc()
+
+    def write(self, path, raw):
+        with lopen(path, 'wb') as f:
+            f.write(raw)
+
+
+# }}}
+
+class CoverCache(Thread): # {{{
+
+    def __init__(self, db, cover_func):
+        Thread.__init__(self)
+        self.daemon = True
+        self.db = db
+        self.cover_func = cover_func
         self.load_queue = Queue()
         self.keep_running = True
         self.cache = {}
@@ -37,7 +135,7 @@ class CoverCache(Thread):
         self.keep_running = False
 
     def _image_for_id(self, id_):
-        img = self.db.cover(id_, index_is_id=True, as_image=True)
+        img = self.cover_func(id_, index_is_id=True, as_image=True)
         if img is None:
             img = QImage()
         if not img.isNull():
@@ -52,17 +150,46 @@ class CoverCache(Thread):
     def run(self):
         while self.keep_running:
             try:
-                id_ = self.load_queue.get(True, 1)
+                # The GUI puts the same ID into the queue many times. The code
+                # below emptys the queue, building a set of unique values. When
+                # the queue is empty, do the work
+                ids = set()
+                id_ = self.load_queue.get(True, 2)
+                ids.add(id_)
+                try:
+                    while True:
+                        # Give the gui some time to put values into the queue
+                        id_ = self.load_queue.get(True, 0.5)
+                        ids.add(id_)
+                except Empty:
+                    pass
+                except:
+                    # Happens during shutdown
+                    break
             except Empty:
                 continue
-            try:
-                img = self._image_for_id(id_)
             except:
-                import traceback
-                traceback.print_exc()
-                continue
-            with self.lock:
-                self.cache[id_] = img
+                #Happens during interpreter shutdown
+                break
+            if not self.keep_running:
+                break
+            for id_ in ids:
+                time.sleep(0.050) # Limit 20/second to not overwhelm the GUI
+                try:
+                    img = self._image_for_id(id_)
+                except:
+                    try:
+                        traceback.print_exc()
+                    except:
+                        # happens during shutdown
+                        break
+                    continue
+                try:
+                    with self.lock:
+                        self.cache[id_] = img
+                except:
+                    # Happens during interpreter shutdown
+                    break
 
     def set_cache(self, ids):
         with self.lock:
@@ -88,6 +215,7 @@ class CoverCache(Thread):
             for id_ in ids:
                 self.cache.pop(id_, None)
                 self.load_queue.put(id_)
+# }}}
 
 ### Global utility function for get_match here and in gui2/library.py
 CONTAINS_MATCH = 0
@@ -105,14 +233,14 @@ def _match(query, value, matchkind):
             pass
     return False
 
-class ResultCache(SearchQueryParser):
+class ResultCache(SearchQueryParser): # {{{
 
     '''
     Stores sorted and filtered metadata in memory.
     '''
     def __init__(self, FIELD_MAP, field_metadata):
         self.FIELD_MAP = FIELD_MAP
-        self._map = self._map_filtered = self._data = []
+        self._map = self._data = self._map_filtered = []
         self.first_sort = True
         self.search_restriction = ''
         self.field_metadata = field_metadata
@@ -121,6 +249,11 @@ class ResultCache(SearchQueryParser):
         self.build_date_relop_dict()
         self.build_numeric_relop_dict()
 
+        self.composites = []
+        for key in field_metadata:
+            if field_metadata[key]['datatype'] == 'composite':
+                self.composites.append((key, field_metadata[key]['rec_index']))
+
     def __getitem__(self, row):
         return self._data[self._map_filtered[row]]
 
@@ -141,6 +274,8 @@ class ResultCache(SearchQueryParser):
         for x in self.iterall():
             yield x[idx]
 
+    # Search functions {{{
+
     def universal_set(self):
         return set([i[0] for i in self._data if i is not None])
 
@@ -245,7 +380,7 @@ class ResultCache(SearchQueryParser):
             field_count = 3
         else:
             try:
-                qd = parse_date(query)
+                qd = parse_date(query, as_utc=False)
             except:
                 raise ParseException(query, len(query), 'Date conversion error', self)
             if '-' in query:
@@ -324,7 +459,7 @@ class ResultCache(SearchQueryParser):
         if query and query.strip():
             # get metadata key associated with the search term. Eliminates
             # dealing with plurals and other aliases
-            location = self.field_metadata.search_term_to_key(location.lower().strip())
+            location = self.field_metadata.search_term_to_field_key(location.lower().strip())
             if isinstance(location, list):
                 if allow_recursion:
                     for loc in location:
@@ -370,7 +505,7 @@ class ResultCache(SearchQueryParser):
                 if len(self.field_metadata[x]['search_terms']):
                     db_col[x] = self.field_metadata[x]['rec_index']
                     if self.field_metadata[x]['datatype'] not in \
-                                                ['text', 'comments', 'series']:
+                                    ['composite', 'text', 'comments', 'series']:
                         exclude_fields.append(db_col[x])
                     col_datatype[db_col[x]] = self.field_metadata[x]['datatype']
                     is_multiple_cols[db_col[x]] = self.field_metadata[x]['is_multiple']
@@ -462,15 +597,51 @@ class ResultCache(SearchQueryParser):
                             continue
         return matches
 
+    def search(self, query, return_matches=False):
+        ans = self.search_getting_ids(query, self.search_restriction)
+        if return_matches:
+            return ans
+        self._map_filtered = ans
+
+    def search_getting_ids(self, query, search_restriction):
+        q = ''
+        if not query or not query.strip():
+            q = search_restriction
+        else:
+            q = query
+            if search_restriction:
+                q = u'%s (%s)' % (search_restriction, query)
+        if not q:
+            return list(self._map)
+        matches = self.parse(q)
+        tmap = list(itertools.repeat(False, len(self._data)))
+        for x in matches:
+            tmap[x] = True
+        return [x for x in self._map if tmap[x]]
+
+    def set_search_restriction(self, s):
+        self.search_restriction = s
+
+    # }}}
+
     def remove(self, id):
-        self._data[id] = None
-        if id in self._map:
+        try:
+            self._data[id] = None
+        except IndexError:
+            # id is out of bounds, no point setting it to None anyway
+            pass
+        try:
             self._map.remove(id)
-        if id in self._map_filtered:
+        except ValueError:
+            pass
+        try:
             self._map_filtered.remove(id)
+        except ValueError:
+            pass
 
     def set(self, row, col, val, row_is_id=False):
         id = row if row_is_id else self._map_filtered[row]
+        self._data[id][self.FIELD_MAP['all_metadata']] = None
         self._data[id][col] = val
 
     def get(self, row, col, row_is_id=False):
@@ -501,6 +672,11 @@ class ResultCache(SearchQueryParser):
                 self._data[id] = db.conn.get('SELECT * from meta2 WHERE id=?', (id,))[0]
                 self._data[id].append(db.has_cover(id, index_is_id=True))
                 self._data[id].append(db.book_on_device_string(id))
+                self._data[id].append(None)
+                if len(self.composites) > 0:
+                    mi = db.get_metadata(id, index_is_id=True)
+                    for k,c in self.composites:
+                        self._data[id][c] = mi.get(k, None)
             except IndexError:
                 return None
         try:
@@ -517,14 +693,17 @@ class ResultCache(SearchQueryParser):
             self._data[id] = db.conn.get('SELECT * from meta2 WHERE id=?', (id,))[0]
             self._data[id].append(db.has_cover(id, index_is_id=True))
             self._data[id].append(db.book_on_device_string(id))
+            self._data[id].append(None)
+            if len(self.composites) > 0:
+                mi = db.get_metadata(id, index_is_id=True)
+                for k,c in self.composites:
+                    self._data[id][c] = mi.get(k)
         self._map[0:0] = ids
         self._map_filtered[0:0] = ids
 
     def books_deleted(self, ids):
         for id in ids:
-            self._data[id] = None
-            if id in self._map: self._map.remove(id)
-            if id in self._map_filtered: self._map_filtered.remove(id)
+            self.remove(id)
 
     def count(self):
         return len(self._map)
@@ -544,95 +723,110 @@ class ResultCache(SearchQueryParser):
             if item is not None:
                 item.append(db.has_cover(item[0], index_is_id=True))
                 item.append(db.book_on_device_string(item[0]))
+                item.append(None)
+                if len(self.composites) > 0:
+                    mi = db.get_metadata(item[0], index_is_id=True)
+                    for k,c in self.composites:
+                        item[c] = mi.get(k)
+
         self._map = [i[0] for i in self._data if i is not None]
         if field is not None:
             self.sort(field, ascending)
         self._map_filtered = list(self._map)
         if self.search_restriction:
-            self.search('', return_matches=False, ignore_search_restriction=False)
+            self.search('', return_matches=False)
 
-    def seriescmp(self, sidx, siidx, x, y, library_order=None):
-        try:
-            if library_order:
-                ans = cmp(title_sort(self._data[x][sidx].lower()),
-                                title_sort(self._data[y][sidx].lower()))
-            else:
-                ans = cmp(self._data[x][sidx].lower(),
-                                                self._data[y][sidx].lower())
-        except AttributeError: # Some entries may be None
-            ans = cmp(self._data[x][sidx], self._data[y][sidx])
-        if ans != 0: return ans
-        return cmp(self._data[x][siidx], self._data[y][siidx])
+    # Sorting functions {{{
 
-    def cmp(self, loc, x, y, asstr=True, subsort=False):
-        try:
-            ans = cmp(self._data[x][loc].lower(), self._data[y][loc].lower()) if \
-                asstr else cmp(self._data[x][loc], self._data[y][loc])
-        except AttributeError: # Some entries may be None
-            ans = cmp(self._data[x][loc], self._data[y][loc])
-        except TypeError: ## raised when a datetime is None
-            x = self._data[x][loc]
-            if x is None:
-                x = UNDEFINED_DATE
-            y = self._data[y][loc]
-            if y is None:
-                y = UNDEFINED_DATE
-            return cmp(x, y)
-        if subsort and ans == 0:
-            return cmp(self._data[x][11].lower(), self._data[y][11].lower())
-        return ans
+    def sanitize_sort_field_name(self, field):
+        field = self.field_metadata.search_term_to_field_key(field.lower().strip())
+        # translate some fields to their hidden equivalent
+        if field == 'title': field = 'sort'
+        elif field == 'authors': field = 'author_sort'
+        return field
 
     def sort(self, field, ascending, subsort=False):
-        field = field.lower().strip()
-        if field in ('author', 'tag', 'comment'):
-            field += 's'
-        if   field == 'date': field = 'timestamp'
-        elif field == 'title': field = 'sort'
-        elif field == 'authors': field = 'author_sort'
-        as_string = field not in ('size', 'rating', 'timestamp')
+        self.multisort([(field, ascending)])
 
-        if self.first_sort:
-            subsort = True
-            self.first_sort = False
-        if self.field_metadata[field]['is_custom']:
-            if self.field_metadata[field]['datatype'] == 'series':
-                fcmp = functools.partial(self.seriescmp,
-                    self.field_metadata[field]['rec_index'],
-                    self.field_metadata.cc_series_index_column_for(field),
-                    library_order=tweaks['title_series_sorting'] == 'library_order')
-            else:
-                as_string = self.field_metadata[field]['datatype'] in ('comments', 'text')
-                field = self.field_metadata[field]['colnum']
-                fcmp = functools.partial(self.cmp, self.FIELD_MAP[field],
-                                     subsort=subsort, asstr=as_string)
-        elif field == 'series':
-            fcmp = functools.partial(self.seriescmp, self.FIELD_MAP['series'],
-                self.FIELD_MAP['series_index'],
-                library_order=tweaks['title_series_sorting'] == 'library_order')
+    def multisort(self, fields=[], subsort=False):
+        fields = [(self.sanitize_sort_field_name(x), bool(y)) for x, y in fields]
+        keys = self.field_metadata.sortable_field_keys()
+        fields = [x for x in fields if x[0] in keys]
+        if subsort and 'sort' not in [x[0] for x in fields]:
+            fields += [('sort', True)]
+        if not fields:
+            fields = [('timestamp', False)]
+
+        keyg = SortKeyGenerator(fields, self.field_metadata, self._data)
+        if len(fields) == 1:
+            self._map.sort(key=keyg, reverse=not fields[0][1])
         else:
-            fcmp = functools.partial(self.cmp, self.FIELD_MAP[field],
-                                     subsort=subsort, asstr=as_string)
-        self._map.sort(cmp=fcmp, reverse=not ascending)
-        self._map_filtered = [id for id in self._map if id in self._map_filtered]
+            self._map.sort(key=keyg)
 
-    def search(self, query, return_matches=False):
-        ans = self.search_getting_ids(query, self.search_restriction)
-        if return_matches:
-            return ans
-        self._map_filtered = ans
+        tmap = list(itertools.repeat(False, len(self._data)))
+        for x in self._map_filtered:
+            tmap[x] = True
+        self._map_filtered = [x for x in self._map if tmap[x]]
 
-    def search_getting_ids(self, query, search_restriction):
-        q = ''
-        if not query or not query.strip():
-            q = search_restriction
-        else:
-            q = query
-            if search_restriction:
-                q = u'%s (%s)' % (search_restriction, query)
-        if not q:
-            return list(self._map)
-        matches = sorted(self.parse(q))
-        return [id for id in self._map if id in matches]
 
-    def set_search_restriction(self, s):
-        self.search_restriction = s
+class SortKey(object):
+
+    def __init__(self, orders, values):
+        self.orders, self.values = orders, values
+
+    def __cmp__(self, other):
+        for i, ascending in enumerate(self.orders):
+            ans = cmp(self.values[i], other.values[i])
+            if ans != 0:
+                return ans * ascending
+        return 0
+
+class SortKeyGenerator(object):
+
+    def __init__(self, fields, field_metadata, data):
+        self.field_metadata = field_metadata
+        self.orders = [-1 if x[1] else 1 for x in fields]
+        self.entries = [(x[0], field_metadata[x[0]]) for x in fields]
+        self.library_order = tweaks['title_series_sorting'] == 'library_order'
+        self.data = data
+
+    def __call__(self, record):
+        values = tuple(self.itervals(self.data[record]))
+        if len(values) == 1:
+            return values[0]
+        return SortKey(self.orders, values)
+
+    def itervals(self, record):
+        for name, fm in self.entries:
+            dt = fm['datatype']
+            val = record[fm['rec_index']]
+
+            if dt == 'datetime':
+                if val is None:
+                    val = UNDEFINED_DATE
+
+            elif dt == 'series':
+                if val is None:
+                    val = ('', 1)
+                else:
+                    val = val.lower()
+                    if self.library_order:
+                        val = title_sort(val)
+                    sidx_fm = self.field_metadata[name + '_index']
+                    sidx = record[sidx_fm['rec_index']]
+                    val = (val, sidx)
+
+            elif dt in ('text', 'comments', 'composite'):
+                if val is None:
+                    val = ''
+                val = val.lower()
+
+            elif dt == 'bool':
+                val = {True: 1, False: 2, None: 3}.get(val, 3)
+
+            yield val
+
+    # }}}
+
+# }}}
+
