@@ -5,18 +5,23 @@ __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import re, os, cStringIO, operator
+import re, os
 
 import cherrypy
-try:
-    from PIL import Image as PILImage
-    PILImage
-except ImportError:
-    import Image as PILImage
 
 from calibre import fit_image, guess_type
 from calibre.utils.date import fromtimestamp
-from calibre.ebooks.metadata import title_sort
+from calibre.library.caches import SortKeyGenerator
+from calibre.utils.magick.draw import save_cover_data_to, Image, \
+        thumbnail as generate_thumbnail
+
+class CSSortKeyGenerator(SortKeyGenerator):
+
+    def __init__(self, fields, fm):
+        SortKeyGenerator.__init__(self, fields, fm, None)
+
+    def __call__(self, record):
+        return self.itervals(record).next()
 
 class ContentServer(object):
 
@@ -27,9 +32,10 @@ class ContentServer(object):
 
     def add_routes(self, connect):
         connect('root', '/', self.index)
+        connect('old', '/old', self.old)
         connect('get', '/get/{what}/{id}', self.get,
                 conditions=dict(method=["GET", "HEAD"]))
-        connect('static', '/static/{name}', self.static,
+        connect('static', '/static/{name:.*?}', self.static,
                 conditions=dict(method=["GET", "HEAD"]))
 
     # Utility methods {{{
@@ -47,32 +53,12 @@ class ContentServer(object):
 
 
     def sort(self, items, field, order):
-        field = field.lower().strip()
-        if field == 'author':
-            field = 'authors'
-        if field == 'date':
-            field = 'timestamp'
-        if field not in ('title', 'authors', 'rating', 'timestamp', 'tags', 'size', 'series'):
+        field = self.db.data.sanitize_sort_field_name(field)
+        if field not in self.db.field_metadata.sortable_field_keys():
             raise cherrypy.HTTPError(400, '%s is not a valid sort field'%field)
-        cmpf = cmp if field in ('rating', 'size', 'timestamp') else \
-                lambda x, y: cmp(x.lower() if x else '', y.lower() if y else '')
-        if field == 'series':
-            items.sort(cmp=self.seriescmp, reverse=not order)
-        else:
-            lookup = 'sort' if field == 'title' else field
-            lookup = 'author_sort' if field == 'authors' else field
-            field = self.db.FIELD_MAP[lookup]
-            getter = operator.itemgetter(field)
-            items.sort(cmp=lambda x, y: cmpf(getter(x), getter(y)), reverse=not order)
+        keyg = CSSortKeyGenerator([(field, order)], self.db.field_metadata)
+        items.sort(key=keyg, reverse=not order)
 
-    def seriescmp(self, x, y):
-        si = self.db.FIELD_MAP['series']
-        try:
-            ans = cmp(title_sort(x[si].lower()), title_sort(y[si].lower()))
-        except AttributeError: # Some entries may be None
-            ans = cmp(x[si], y[si])
-        if ans != 0: return ans
-        return cmp(x[self.db.FIELD_MAP['series_index']], y[self.db.FIELD_MAP['series_index']])
     # }}}
 
 
@@ -84,12 +70,17 @@ class ContentServer(object):
             id = id.rpartition('_')[-1].partition('.')[0]
             match = re.search(r'\d+', id)
             if not match:
-                raise cherrypy.HTTPError(400, 'id:%s not an integer'%id)
+                raise cherrypy.HTTPError(404, 'id:%s not an integer'%id)
             id = int(match.group())
         if not self.db.has_id(id):
-            raise cherrypy.HTTPError(400, 'id:%d does not exist in database'%id)
-        if what == 'thumb':
-            return self.get_cover(id, thumbnail=True)
+            raise cherrypy.HTTPError(404, 'id:%d does not exist in database'%id)
+        if what == 'thumb' or what.startswith('thumb_'):
+            try:
+                width, height = map(int, what.split('_')[1:])
+            except:
+                width, height = 60, 80
+            return self.get_cover(id, thumbnail=True, thumb_width=width,
+                    thumb_height=height)
         if what == 'cover':
             return self.get_cover(id)
         return self.get_format(id, what)
@@ -112,7 +103,11 @@ class ContentServer(object):
         if self.opts.develop:
             lm = fromtimestamp(os.stat(path).st_mtime)
             cherrypy.response.headers['Last-Modified'] = self.last_modified(lm)
-        return open(path, 'rb').read()
+        with open(path, 'rb') as f:
+            ans = f.read()
+        if path.endswith('.css'):
+            ans = ans.replace('/static/', self.opts.url_prefix + '/static/')
+        return ans
 
     def index(self, **kwargs):
         'The / URL'
@@ -133,42 +128,46 @@ class ContentServer(object):
         if want_mobile:
             return self.mobile()
 
-        return self.static('index.html')
-
+        return self.browse_catalog()
 
+    def old(self, **kwargs):
+        return self.static('index.html').replace('{prefix}',
+                self.opts.url_prefix)
 
     # Actually get content from the database {{{
-    def get_cover(self, id, thumbnail=False):
-        cover = self.db.cover(id, index_is_id=True, as_file=False)
-        if cover is None:
-            cover = self.default_cover
-        cherrypy.response.headers['Content-Type'] = 'image/jpeg'
-        cherrypy.response.timeout = 3600
-        path = getattr(cover, 'name', False)
-        updated = fromtimestamp(os.stat(path).st_mtime) if path and \
-            os.access(path, os.R_OK) else self.build_time
-        cherrypy.response.headers['Last-Modified'] = self.last_modified(updated)
+    def get_cover(self, id, thumbnail=False, thumb_width=60, thumb_height=80):
         try:
-            f = cStringIO.StringIO(cover)
-            try:
-                im = PILImage.open(f)
-            except IOError:
-                raise cherrypy.HTTPError(404, 'No valid cover found')
-            width, height = im.size
+            cherrypy.response.headers['Content-Type'] = 'image/jpeg'
+            cherrypy.response.timeout = 3600
+            cover = self.db.cover(id, index_is_id=True, as_file=True)
+            if cover is None:
+                cover = self.default_cover
+                updated = self.build_time
+            else:
+                with cover as f:
+                    updated = fromtimestamp(os.fstat(f.fileno()).st_mtime)
+                    cover = f.read()
+            cherrypy.response.headers['Last-Modified'] = self.last_modified(updated)
+
+            if thumbnail:
+                return generate_thumbnail(cover,
+                        width=thumb_width, height=thumb_height)[-1]
+
+            img = Image()
+            img.load(cover)
+            width, height = img.size
             scaled, width, height = fit_image(width, height,
-                60 if thumbnail else self.max_cover_width,
-                80 if thumbnail else self.max_cover_height)
+                thumb_width if thumbnail else self.max_cover_width,
+                thumb_height if thumbnail else self.max_cover_height)
             if not scaled:
                 return cover
-            im = im.resize((int(width), int(height)), PILImage.ANTIALIAS)
-            of = cStringIO.StringIO()
-            im.convert('RGB').save(of, 'JPEG')
-            return of.getvalue()
+            return save_cover_data_to(img, 'img.jpg', return_data=True,
+                    resize_to=(width, height))
         except Exception, err:
             import traceback
             cherrypy.log.error('Failed to generate cover:')
             cherrypy.log.error(traceback.print_exc())
-            raise cherrypy.HTTPError(404, 'Failed to generate cover: %s'%err)
+            raise cherrypy.HTTPError(404, 'Failed to generate cover: %r'%err)
 
     def get_format(self, id, format):
         format = format.upper()
@@ -196,7 +195,7 @@ class ContentServer(object):
         if path and os.path.exists(path):
             updated = fromtimestamp(os.stat(path).st_mtime)
             cherrypy.response.headers['Last-Modified'] = self.last_modified(updated)
-        return fmt.read()
+        return fmt
     # }}}
 
 
