@@ -17,9 +17,10 @@ import routes
 from calibre.constants import __appname__
 from calibre.ebooks.metadata import fmt_sidx
 from calibre.library.comments import comments_to_html
+from calibre.library.server import custom_fields_to_display
+from calibre.library.server.utils import format_tag_string, Offsets
 from calibre import guess_type
 from calibre.utils.ordered_dict import OrderedDict
-from calibre.utils.date import format_date
 
 BASE_HREFS = {
         0 : '/stanza',
@@ -131,7 +132,8 @@ def CATALOG_GROUP_ENTRY(item, category, base_href, version, updated):
             link
             )
 
-def ACQUISITION_ENTRY(item, version, FM, updated, CFM, CKEYS):
+def ACQUISITION_ENTRY(item, version, db, updated, CFM, CKEYS, prefix):
+    FM = db.FIELD_MAP
     title = item[FM['title']]
     if not title:
         title = _('Unknown')
@@ -147,28 +149,25 @@ def ACQUISITION_ENTRY(item, version, FM, updated, CFM, CKEYS):
         extra.append(_('RATING: %s<br />')%rating)
     tags = item[FM['tags']]
     if tags:
-        extra.append(_('TAGS: %s<br />')%\
-                ', '.join(tags.split(',')))
+        extra.append(_('TAGS: %s<br />')%format_tag_string(tags, ',',
+                                                           ignore_max=True,
+                                                           no_tag_count=True))
     series = item[FM['series']]
     if series:
         extra.append(_('SERIES: %s [%s]<br />')%\
                 (series,
                 fmt_sidx(float(item[FM['series_index']]))))
     for key in CKEYS:
-        val = item[CFM[key]['rec_index']]
-        if val is not None:
-            name = CFM[key]['name']
+        mi = db.get_metadata(item[CFM['id']['rec_index']], index_is_id=True)
+        name, val = mi.format_field(key)
+        if val:
             datatype = CFM[key]['datatype']
             if datatype == 'text' and CFM[key]['is_multiple']:
-                extra.append('%s: %s<br />'%(name, ', '.join(val.split('|'))))
-            elif datatype == 'series':
-                extra.append('%s: %s [%s]<br />'%(name, val,
-                              fmt_sidx(item[CFM.cc_series_index_column_for(key)])))
-            elif datatype == 'datetime':
-                extra.append('%s: %s<br />'%(name,
-                    format_date(val, CFM[key]['display'].get('date_format','dd MMM yyyy'))))
+                extra.append('%s: %s<br />'%(name, format_tag_string(val, ',',
+                                                           ignore_max=True,
+                                                           no_tag_count=True)))
             else:
-                extra.append('%s: %s <br />' % (CFM[key]['name'], val))
+                extra.append('%s: %s<br />'%(name, val))
     comments = item[FM['comments']]
     if comments:
         comments = comments_to_html(comments)
@@ -186,16 +185,16 @@ def ACQUISITION_ENTRY(item, version, FM, updated, CFM, CKEYS):
         for fmt in formats.split(','):
             fmt = fmt.lower()
             mt = guess_type('a.'+fmt)[0]
-            href = '/get/%s/%s'%(fmt, item[FM['id']])
+            href = prefix + '/get/%s/%s'%(fmt, item[FM['id']])
             if mt:
                 link = E.link(type=mt, href=href)
                 if version > 0:
                     link.set('rel', "http://opds-spec.org/acquisition")
                 ans.append(link)
-    ans.append(E.link(type='image/jpeg', href='/get/cover/%s'%item[FM['id']],
+    ans.append(E.link(type='image/jpeg', href=prefix+'/get/cover/%s'%item[FM['id']],
         rel="x-stanza-cover-image" if version == 0 else
         "http://opds-spec.org/cover"))
-    ans.append(E.link(type='image/jpeg', href='/get/thumb/%s'%item[FM['id']],
+    ans.append(E.link(type='image/jpeg', href=prefix+'/get/thumb/%s'%item[FM['id']],
         rel="x-stanza-cover-image-thumbnail" if version == 0 else
         "http://opds-spec.org/thumbnail"))
 
@@ -276,14 +275,15 @@ class NavFeed(Feed):
 class AcquisitionFeed(NavFeed):
 
     def __init__(self, updated, id_, items, offsets, page_url, up_url, version,
-            FM, CFM):
+            db, prefix):
         NavFeed.__init__(self, id_, updated, version, offsets, page_url, up_url)
-        CKEYS = [key for key in sorted(CFM.get_custom_fields(),
+        CFM = db.field_metadata
+        CKEYS = [key for key in sorted(custom_fields_to_display(db),
                  cmp=lambda x,y: cmp(CFM[x]['name'].lower(),
                                      CFM[y]['name'].lower()))]
         for item in items:
-            self.root.append(ACQUISITION_ENTRY(item, version, FM, updated,
-                                               CFM, CKEYS))
+            self.root.append(ACQUISITION_ENTRY(item, version, db, updated,
+                                               CFM, CKEYS, prefix))
 
 class CategoryFeed(NavFeed):
 
@@ -321,26 +321,6 @@ class CategoryGroupFeed(NavFeed):
             self.root.append(CATALOG_GROUP_ENTRY(item, which, base_href, version, updated))
 
 
-class OPDSOffsets(object):
-
-    def __init__(self, offset, delta, total):
-        if offset < 0:
-            offset = 0
-        if offset >= total:
-            raise cherrypy.HTTPError(404, 'Invalid offset: %r'%offset)
-        last_allowed_index = total - 1
-        last_current_index = offset + delta - 1
-        self.offset = offset
-        self.next_offset = last_current_index + 1
-        if self.next_offset > last_allowed_index:
-            self.next_offset = -1
-        self.previous_offset = self.offset - delta
-        if self.previous_offset < 0:
-            self.previous_offset = 0
-        self.last_offset = last_allowed_index - delta
-        if self.last_offset < 0:
-            self.last_offset = 0
-
 
 class OPDSServer(object):
 
@@ -374,13 +354,14 @@ class OPDSServer(object):
         items = [x for x in self.db.data.iterall() if x[idx] in ids]
         self.sort(items, sort_by, ascending)
         max_items = self.opts.max_opds_items
-        offsets = OPDSOffsets(offset, max_items, len(items))
+        offsets = Offsets(offset, max_items, len(items))
         items = items[offsets.offset:offsets.offset+max_items]
         updated = self.db.last_modified()
         cherrypy.response.headers['Last-Modified'] = self.last_modified(updated)
         cherrypy.response.headers['Content-Type'] = 'application/atom+xml;profile=opds-catalog'
         return str(AcquisitionFeed(updated, id_, items, offsets,
-            page_url, up_url, version, self.db.FIELD_MAP, self.db.field_metadata))
+                                   page_url, up_url, version, self.db,
+                                   self.opts.url_prefix))
 
     def opds_search(self, query=None, version=0, offset=0):
         try:
@@ -448,7 +429,7 @@ class OPDSServer(object):
         id_ = 'calibre-category-group-feed:'+category+':'+which
 
         max_items = self.opts.max_opds_items
-        offsets = OPDSOffsets(offset, max_items, len(items))
+        offsets = Offsets(offset, max_items, len(items))
         items = list(items)[offsets.offset:offsets.offset+max_items]
 
         cherrypy.response.headers['Last-Modified'] = self.last_modified(updated)
@@ -495,7 +476,7 @@ class OPDSServer(object):
 
         if len(items) <= MAX_ITEMS:
             max_items = self.opts.max_opds_items
-            offsets = OPDSOffsets(offset, max_items, len(items))
+            offsets = Offsets(offset, max_items, len(items))
             items = list(items)[offsets.offset:offsets.offset+max_items]
             ans = CategoryFeed(items, which, id_, updated, version, offsets,
                 page_url, up_url, self.db)
@@ -516,7 +497,7 @@ class OPDSServer(object):
                     getattr(y, 'sort', y.name).startswith(x)])
             items = [Group(x, y) for x, y in category_groups.items()]
             max_items = self.opts.max_opds_items
-            offsets = OPDSOffsets(offset, max_items, len(items))
+            offsets = Offsets(offset, max_items, len(items))
             items = items[offsets.offset:offsets.offset+max_items]
             ans = CategoryGroupFeed(items, which, id_, updated, version, offsets,
                 page_url, up_url)
