@@ -414,6 +414,24 @@ magick_Image_load(magick_Image *self, PyObject *args, PyObject *kwargs) {
 
 // }}}
 
+// Image.load {{{
+static PyObject *
+magick_Image_read(magick_Image *self, PyObject *args, PyObject *kwargs) {
+    const char *data;
+    MagickBooleanType res;
+    
+    if (!PyArg_ParseTuple(args, "s", &data)) return NULL;
+
+    res = MagickReadImage(self->wand, data);
+
+    if (!res)
+        return magick_set_exception(self->wand);
+
+    Py_RETURN_NONE;
+}
+
+// }}}
+
 // Image.create_canvas {{{
 static PyObject *
 magick_Image_create_canvas(magick_Image *self, PyObject *args, PyObject *kwargs)
@@ -725,6 +743,49 @@ magick_Image_set_page(magick_Image *self, PyObject *args, PyObject *kwargs) {
 }
 // }}}
 
+// Image.set_compression_quality {{{
+
+static PyObject *
+magick_Image_set_compression_quality(magick_Image *self, PyObject *args, PyObject *kwargs) {
+    Py_ssize_t quality;
+    
+    if (!PyArg_ParseTuple(args, "n", &quality)) return NULL;
+
+    if (!MagickSetImageCompressionQuality(self->wand, quality)) return magick_set_exception(self->wand);
+
+    Py_RETURN_NONE;
+}
+// }}}
+
+// Image.has_transparent_pixels {{{
+
+static PyObject *
+magick_Image_has_transparent_pixels(magick_Image *self, PyObject *args, PyObject *kwargs) {
+    PixelIterator *pi = NULL;
+    PixelWand **pixels = NULL;
+    int found = 0;
+    size_t r, c, width, height;
+    double alpha;
+
+    height = MagickGetImageHeight(self->wand);
+    pi = NewPixelIterator(self->wand);
+
+    for (r = 0; r < height; r++) {
+        pixels = PixelGetNextIteratorRow(pi, &width);
+        for (c = 0; c < width; c++) {
+            alpha = PixelGetAlpha(pixels[c]);
+            if (alpha < 1.00) {
+                found = 1;
+                c = width; r = height;
+            }
+        }
+    }
+    pi = DestroyPixelIterator(pi);
+    if (found) Py_RETURN_TRUE;
+    Py_RETURN_FALSE;
+}
+// }}}
+
 // Image.normalize {{{
 
 static PyObject *
@@ -830,6 +891,10 @@ static PyMethodDef magick_Image_methods[] = {
      "Load an image from a byte buffer (string)"
     },
 
+    {"read", (PyCFunction)magick_Image_read, METH_VARARGS,
+     "Read image from path. Path must be a bytestring in the filesystem encoding"
+    },
+
     {"export", (PyCFunction)magick_Image_export, METH_VARARGS,
      "export(format) -> bytestring\n\n Export the image as the specified format"
     },
@@ -872,6 +937,14 @@ static PyMethodDef magick_Image_methods[] = {
      "set_page(width, height, x, y) \n\n Sets the page geometry of the image."
     },
 
+    {"set_compression_quality", (PyCFunction)magick_Image_set_compression_quality, METH_VARARGS,
+     "set_compression_quality(quality) \n\n Sets the compression quality when exporting the image."
+    },
+
+    {"has_transparent_pixels", (PyCFunction)magick_Image_has_transparent_pixels, METH_VARARGS,
+     "has_transparent_pixels() \n\n Returns True iff image has a (semi-) transparent pixel"
+    },
+
     {"thumbnail", (PyCFunction)magick_Image_thumbnail, METH_VARARGS,
      "thumbnail(width, height) \n\n Convert to a thumbnail of specified size."
     },
