@@ -7,7 +7,7 @@ Defines various abstract base classes that can be subclassed to create powerful
 __docformat__ = "restructuredtext en"
 
 
-import os, time, traceback, re, urlparse, sys
+import os, time, traceback, re, urlparse, sys, cStringIO
 from collections import defaultdict
 from functools import partial
 from contextlib import nested, closing
@@ -27,6 +27,7 @@ from calibre.web.fetch.simple import RecursiveFetcher
 from calibre.utils.threadpool import WorkRequest, ThreadPool, NoResultsPending
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.date import now as nowf
+from calibre.utils.magick.draw import save_cover_data_to, add_borders_to_image
 
 class LoginFailed(ValueError):
     pass
@@ -81,9 +82,6 @@ class BasicNewsRecipe(Recipe):
     #: Automatically reduced to 1 if :attr:`BasicNewsRecipe.delay` > 0
     simultaneous_downloads = 5
 
-    #: If False the remote server is contacted by only one thread at a time
-    multithreaded_fetch = False
-
     #: Timeout for fetching files from server in seconds
     timeout                = 120.0
 
@@ -109,9 +107,11 @@ class BasicNewsRecipe(Recipe):
 
     #: If True the GUI will ask the user for a username and password
     #: to use while downloading
-    #: @type: boolean
+    #: If set to "optional" the use of a username and password becomes optional
     needs_subscription     = False
 
+    #:
+
     #: If True the navigation bar is center aligned, otherwise it is left aligned
     center_navbar = True
 
@@ -290,10 +290,12 @@ class BasicNewsRecipe(Recipe):
     #: the cover for the periodical.  Overriding this in your recipe instructs
     #: calibre to render the downloaded cover into a frame whose width and height
     #: are expressed as a percentage of the downloaded cover.
-    #: cover_margins = (10,15,'white') pads the cover with a white margin
+    #: cover_margins = (10, 15, '#ffffff') pads the cover with a white margin
     #: 10px on the left and right, 15px on the top and bottom.
-    #: Colors name defined at http://www.imagemagick.org/script/color.php
-    cover_margins = (0,0,'white')
+    #: Color names defined at http://www.imagemagick.org/script/color.php
+    #: Note that for some reason, white does not always work on windows. Use
+    #: #ffffff instead
+    cover_margins = (0, 0, '#ffffff')
 
     #: Set to a non empty string to disable this recipe
     #: The string will be used as the disabled message
@@ -397,6 +399,23 @@ class BasicNewsRecipe(Recipe):
         '''
         return browser(*args, **kwargs)
 
+    def clone_browser(self, br):
+        '''
+        Clone the browser br. Cloned browsers are used for multi-threaded
+        downloads, since mechanize is not thread safe. The default cloning
+        routines should capture most browser customization, but if you do
+        something exotic in your recipe, you should override this method in
+        your recipe and clone manually.
+
+        Cloned browser instances use the same, thread-safe CookieJar by
+        default, unless you have customized cookie handling.
+        '''
+        if callable(getattr(br, 'clone_browser', None)):
+            return br.clone_browser()
+
+        # Uh-oh recipe using something exotic, call get_browser
+        return self.get_browser()
+
     def get_article_url(self, article):
         '''
         Override in a subclass to customize extraction of the :term:`URL` that points
@@ -578,7 +597,7 @@ class BasicNewsRecipe(Recipe):
             self.title = unicode(self.title, 'utf-8', 'replace')
 
         self.debug = options.verbose > 1
-        self.output_dir = os.getcwd()
+        self.output_dir = os.path.abspath(os.getcwdu())
         self.verbose = options.verbose
         self.test = options.test
         self.username = options.username
@@ -589,7 +608,6 @@ class BasicNewsRecipe(Recipe):
         if self.touchscreen:
             self.template_css += self.output_profile.touchscreen_news_css
 
-        self.output_dir = os.path.abspath(self.output_dir)
         if options.test:
             self.max_articles_per_feed = 2
             self.simultaneous_downloads = min(4, self.simultaneous_downloads)
@@ -606,7 +624,8 @@ class BasicNewsRecipe(Recipe):
         if self.needs_subscription and (\
                 self.username is None or self.password is None or \
                 (not self.username and not self.password)):
-            raise ValueError(_('The "%s" recipe needs a username and password.')%self.title)
+            if self.needs_subscription != 'optional':
+                raise ValueError(_('The "%s" recipe needs a username and password.')%self.title)
 
         self.browser = self.get_browser()
         self.image_map, self.image_counter = {}, 1
@@ -793,17 +812,23 @@ class BasicNewsRecipe(Recipe):
                               extra_css=css).render(doctype='xhtml')
 
 
-    def _fetch_article(self, url, dir, f, a, num_of_feeds):
-        self.web2disk_options.browser = self.get_browser() if self.multithreaded_fetch else self.browser
+    def _fetch_article(self, url, dir_, f, a, num_of_feeds):
+        br = self.browser
+        if self.get_browser.im_func is BasicNewsRecipe.get_browser.im_func:
+            # We are using the default get_browser, which means no need to
+            # clone
+            br = BasicNewsRecipe.get_browser(self)
+        else:
+            br = self.clone_browser(self.browser)
+        self.web2disk_options.browser = br
         fetcher = RecursiveFetcher(self.web2disk_options, self.log,
                 self.image_map, self.css_map,
                 (url, f, a, num_of_feeds))
-        fetcher.base_dir = dir
-        fetcher.current_dir = dir
+        fetcher.browser = br
+        fetcher.base_dir = dir_
+        fetcher.current_dir = dir_
         fetcher.show_progress = False
         fetcher.image_url_processor = self.image_url_processor
-        if self.multithreaded_fetch:
-            fetcher.browser_lock = fetcher.DUMMY_LOCK
         res, path, failures = fetcher.start_fetch(url), fetcher.downloaded_paths, fetcher.failed_links
         if not res or not os.path.exists(res):
             raise Exception(_('Could not fetch article. Run with -vv to see the reason'))
@@ -836,6 +861,9 @@ class BasicNewsRecipe(Recipe):
         except NotImplementedError:
             feeds = self.parse_feeds()
 
+        if not feeds:
+            raise ValueError('No articles found, aborting')
+
         #feeds = FeedCollection(feeds)
 
         self.report_progress(0, _('Trying to download cover...'))
@@ -946,41 +974,42 @@ class BasicNewsRecipe(Recipe):
         try:
             cu = self.get_cover_url()
         except Exception, err:
-            cu = None
             self.log.error(_('Could not download cover: %s')%str(err))
             self.log.debug(traceback.format_exc())
-        if cu is not None:
-            ext = cu.split('/')[-1].rpartition('.')[-1]
-            if '?' in ext:
-                ext = ''
-            ext = ext.lower() if ext and '/' not in ext else 'jpg'
-            cpath = os.path.join(self.output_dir, 'cover.'+ext)
+        else:
+            if not cu:
+                return
+            cdata = None
             if os.access(cu, os.R_OK):
-                with open(cpath, 'wb') as cfile:
-                    cfile.write(open(cu, 'rb').read())
+                cdata = open(cu, 'rb').read()
             else:
                 self.report_progress(1, _('Downloading cover from %s')%cu)
-                with nested(open(cpath, 'wb'), closing(self.browser.open(cu))) as (cfile, r):
-                    cfile.write(r.read())
-                if self.cover_margins[0] or self.cover_margins[1]:
-                    from calibre.utils.magick.draw import add_borders_to_image
-                    add_borders_to_image(cpath,
-                                         left=self.cover_margins[0],right=self.cover_margins[0],
-                                         top=self.cover_margins[1],bottom=self.cover_margins[1],
-                                         border_color=self.cover_margins[2])
-            if ext.lower() == 'pdf':
+                with closing(self.browser.open(cu)) as r:
+                    cdata = r.read()
+            if not cdata:
+                return
+            ext = cu.split('/')[-1].rpartition('.')[-1].lower().strip()
+            if ext == 'pdf':
                 from calibre.ebooks.metadata.pdf import get_metadata
-                stream = open(cpath, 'rb')
+                stream = cStringIO.StringIO(cdata)
+                cdata = None
                 mi = get_metadata(stream)
-                cpath = None
                 if mi.cover_data and mi.cover_data[1]:
-                    cpath = os.path.join(self.output_dir,
-                            'cover.'+mi.cover_data[0])
-                    with open(cpath, 'wb') as f:
-                        f.write(mi.cover_data[1])
+                    cdata = mi.cover_data[1]
+            if not cdata:
+                return
+            if self.cover_margins[0] or self.cover_margins[1]:
+                cdata = add_borders_to_image(cdata,
+                            left=self.cover_margins[0],right=self.cover_margins[0],
+                            top=self.cover_margins[1],bottom=self.cover_margins[1],
+                            border_color=self.cover_margins[2])
+
+            cpath = os.path.join(self.output_dir, 'cover.jpg')
+            save_cover_data_to(cdata, cpath)
             self.cover_path = cpath
 
     def download_cover(self):
+        self.cover_path = None
         try:
             self._download_cover()
         except:
@@ -1100,7 +1129,7 @@ class BasicNewsRecipe(Recipe):
         mi = MetaInformation(title, [__appname__])
         mi.publisher = __appname__
         mi.author_sort = __appname__
-        mi.publication_type = 'periodical:'+self.publication_type
+        mi.publication_type = 'periodical:'+self.publication_type+':'+self.short_title()
         mi.timestamp = nowf()
         mi.comments = self.description
         if not isinstance(mi.comments, unicode):
@@ -1378,7 +1407,7 @@ class CustomIndexRecipe(BasicNewsRecipe):
     def download(self):
         index = os.path.abspath(self.custom_index())
         url = 'file:'+index if iswindows else 'file://'+index
-        self.web2disk_options.browser = self.browser
+        self.web2disk_options.browser = self.clone_browser(self.browser)
         fetcher = RecursiveFetcher(self.web2disk_options, self.log)
         fetcher.base_dir = self.output_dir
         fetcher.current_dir = self.output_dir
@@ -1420,7 +1449,6 @@ class CalibrePeriodical(BasicNewsRecipe):
         return br
 
     def download(self):
-        import cStringIO
         self.log('Fetching downloaded recipe')
         try:
             raw = self.browser.open_novisit(
